from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="MediaAttachment")


@_attrs_define
class MediaAttachment:
    """Class MediaAttachment.

    Attributes:
        codec (Union[Unset, None, str]): Gets or sets the codec.
        codec_tag (Union[Unset, None, str]): Gets or sets the codec tag.
        comment (Union[Unset, None, str]): Gets or sets the comment.
        index (Union[Unset, int]): Gets or sets the index.
        file_name (Union[Unset, None, str]): Gets or sets the filename.
        mime_type (Union[Unset, None, str]): Gets or sets the MIME type.
        delivery_url (Union[Unset, None, str]): Gets or sets the delivery URL.
    """

    codec: Union[Unset, None, str] = UNSET
    codec_tag: Union[Unset, None, str] = UNSET
    comment: Union[Unset, None, str] = UNSET
    index: Union[Unset, int] = UNSET
    file_name: Union[Unset, None, str] = UNSET
    mime_type: Union[Unset, None, str] = UNSET
    delivery_url: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        codec = self.codec
        codec_tag = self.codec_tag
        comment = self.comment
        index = self.index
        file_name = self.file_name
        mime_type = self.mime_type
        delivery_url = self.delivery_url

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if codec is not UNSET:
            field_dict["Codec"] = codec
        if codec_tag is not UNSET:
            field_dict["CodecTag"] = codec_tag
        if comment is not UNSET:
            field_dict["Comment"] = comment
        if index is not UNSET:
            field_dict["Index"] = index
        if file_name is not UNSET:
            field_dict["FileName"] = file_name
        if mime_type is not UNSET:
            field_dict["MimeType"] = mime_type
        if delivery_url is not UNSET:
            field_dict["DeliveryUrl"] = delivery_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        codec = d.pop("Codec", UNSET)

        codec_tag = d.pop("CodecTag", UNSET)

        comment = d.pop("Comment", UNSET)

        index = d.pop("Index", UNSET)

        file_name = d.pop("FileName", UNSET)

        mime_type = d.pop("MimeType", UNSET)

        delivery_url = d.pop("DeliveryUrl", UNSET)

        media_attachment = cls(
            codec=codec,
            codec_tag=codec_tag,
            comment=comment,
            index=index,
            file_name=file_name,
            mime_type=mime_type,
            delivery_url=delivery_url,
        )

        return media_attachment



from enum import Enum


class GroupRepeatMode(str, Enum):
    REPEATALL = "RepeatAll"
    REPEATNONE = "RepeatNone"
    REPEATONE = "RepeatOne"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="NewGroupRequestDto")


@_attrs_define
class NewGroupRequestDto:
    """Class NewGroupRequestDto.

    Attributes:
        group_name (Union[Unset, str]): Gets or sets the group name.
    """

    group_name: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        group_name = self.group_name

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if group_name is not UNSET:
            field_dict["GroupName"] = group_name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        group_name = d.pop("GroupName", UNSET)

        new_group_request_dto = cls(
            group_name=group_name,
        )

        return new_group_request_dto



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    item_id: str,
    *,
    container: Union[Unset, None, List[str]] = UNSET,
    media_source_id: Union[Unset, None, str] = UNSET,
    device_id: Union[Unset, None, str] = UNSET,
    user_id: Union[Unset, None, str] = UNSET,
    audio_codec: Union[Unset, None, str] = UNSET,
    max_audio_channels: Union[Unset, None, int] = UNSET,
    transcoding_audio_channels: Union[Unset, None, int] = UNSET,
    max_streaming_bitrate: Union[Unset, None, int] = UNSET,
    audio_bit_rate: Union[Unset, None, int] = UNSET,
    start_time_ticks: Union[Unset, None, int] = UNSET,
    transcoding_container: Union[Unset, None, str] = UNSET,
    transcoding_protocol: Union[Unset, None, str] = UNSET,
    max_audio_sample_rate: Union[Unset, None, int] = UNSET,
    max_audio_bit_depth: Union[Unset, None, int] = UNSET,
    enable_remote_media: Union[Unset, None, bool] = UNSET,
    break_on_non_key_frames: Union[Unset, None, bool] = False,
    enable_redirection: Union[Unset, None, bool] = True,
) -> Dict[str, Any]:
    pass

    params: Dict[str, Any] = {}
    json_container: Union[Unset, None, List[str]] = UNSET
    if not isinstance(container, Unset):
        if container is None:
            json_container = None
        else:
            json_container = container

    params["container"] = json_container

    params["mediaSourceId"] = media_source_id

    params["deviceId"] = device_id

    params["userId"] = user_id

    params["audioCodec"] = audio_codec

    params["maxAudioChannels"] = max_audio_channels

    params["transcodingAudioChannels"] = transcoding_audio_channels

    params["maxStreamingBitrate"] = max_streaming_bitrate

    params["audioBitRate"] = audio_bit_rate

    params["startTimeTicks"] = start_time_ticks

    params["transcodingContainer"] = transcoding_container

    params["transcodingProtocol"] = transcoding_protocol

    params["maxAudioSampleRate"] = max_audio_sample_rate

    params["maxAudioBitDepth"] = max_audio_bit_depth

    params["enableRemoteMedia"] = enable_remote_media

    params["breakOnNonKeyFrames"] = break_on_non_key_frames

    params["enableRedirection"] = enable_redirection

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "head",
        "url": "/Audio/{itemId}/universal".format(
            itemId=item_id,
        ),
        "params": params,
    }


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if response.status_code == HTTPStatus.FOUND:
        return None
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        return None
    if response.status_code == HTTPStatus.FORBIDDEN:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    item_id: str,
    *,
    client: AuthenticatedClient,
    container: Union[Unset, None, List[str]] = UNSET,
    media_source_id: Union[Unset, None, str] = UNSET,
    device_id: Union[Unset, None, str] = UNSET,
    user_id: Union[Unset, None, str] = UNSET,
    audio_codec: Union[Unset, None, str] = UNSET,
    max_audio_channels: Union[Unset, None, int] = UNSET,
    transcoding_audio_channels: Union[Unset, None, int] = UNSET,
    max_streaming_bitrate: Union[Unset, None, int] = UNSET,
    audio_bit_rate: Union[Unset, None, int] = UNSET,
    start_time_ticks: Union[Unset, None, int] = UNSET,
    transcoding_container: Union[Unset, None, str] = UNSET,
    transcoding_protocol: Union[Unset, None, str] = UNSET,
    max_audio_sample_rate: Union[Unset, None, int] = UNSET,
    max_audio_bit_depth: Union[Unset, None, int] = UNSET,
    enable_remote_media: Union[Unset, None, bool] = UNSET,
    break_on_non_key_frames: Union[Unset, None, bool] = False,
    enable_redirection: Union[Unset, None, bool] = True,
) -> Response[Any]:
    """Gets an audio stream.

    Args:
        item_id (str):
        container (Union[Unset, None, List[str]]):
        media_source_id (Union[Unset, None, str]):
        device_id (Union[Unset, None, str]):
        user_id (Union[Unset, None, str]):
        audio_codec (Union[Unset, None, str]):
        max_audio_channels (Union[Unset, None, int]):
        transcoding_audio_channels (Union[Unset, None, int]):
        max_streaming_bitrate (Union[Unset, None, int]):
        audio_bit_rate (Union[Unset, None, int]):
        start_time_ticks (Union[Unset, None, int]):
        transcoding_container (Union[Unset, None, str]):
        transcoding_protocol (Union[Unset, None, str]):
        max_audio_sample_rate (Union[Unset, None, int]):
        max_audio_bit_depth (Union[Unset, None, int]):
        enable_remote_media (Union[Unset, None, bool]):
        break_on_non_key_frames (Union[Unset, None, bool]):
        enable_redirection (Union[Unset, None, bool]):  Default: True.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        item_id=item_id,
        container=container,
        media_source_id=media_source_id,
        device_id=device_id,
        user_id=user_id,
        audio_codec=audio_codec,
        max_audio_channels=max_audio_channels,
        transcoding_audio_channels=transcoding_audio_channels,
        max_streaming_bitrate=max_streaming_bitrate,
        audio_bit_rate=audio_bit_rate,
        start_time_ticks=start_time_ticks,
        transcoding_container=transcoding_container,
        transcoding_protocol=transcoding_protocol,
        max_audio_sample_rate=max_audio_sample_rate,
        max_audio_bit_depth=max_audio_bit_depth,
        enable_remote_media=enable_remote_media,
        break_on_non_key_frames=break_on_non_key_frames,
        enable_redirection=enable_redirection,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    item_id: str,
    *,
    client: AuthenticatedClient,
    container: Union[Unset, None, List[str]] = UNSET,
    media_source_id: Union[Unset, None, str] = UNSET,
    device_id: Union[Unset, None, str] = UNSET,
    user_id: Union[Unset, None, str] = UNSET,
    audio_codec: Union[Unset, None, str] = UNSET,
    max_audio_channels: Union[Unset, None, int] = UNSET,
    transcoding_audio_channels: Union[Unset, None, int] = UNSET,
    max_streaming_bitrate: Union[Unset, None, int] = UNSET,
    audio_bit_rate: Union[Unset, None, int] = UNSET,
    start_time_ticks: Union[Unset, None, int] = UNSET,
    transcoding_container: Union[Unset, None, str] = UNSET,
    transcoding_protocol: Union[Unset, None, str] = UNSET,
    max_audio_sample_rate: Union[Unset, None, int] = UNSET,
    max_audio_bit_depth: Union[Unset, None, int] = UNSET,
    enable_remote_media: Union[Unset, None, bool] = UNSET,
    break_on_non_key_frames: Union[Unset, None, bool] = False,
    enable_redirection: Union[Unset, None, bool] = True,
) -> Response[Any]:
    """Gets an audio stream.

    Args:
        item_id (str):
        container (Union[Unset, None, List[str]]):
        media_source_id (Union[Unset, None, str]):
        device_id (Union[Unset, None, str]):
        user_id (Union[Unset, None, str]):
        audio_codec (Union[Unset, None, str]):
        max_audio_channels (Union[Unset, None, int]):
        transcoding_audio_channels (Union[Unset, None, int]):
        max_streaming_bitrate (Union[Unset, None, int]):
        audio_bit_rate (Union[Unset, None, int]):
        start_time_ticks (Union[Unset, None, int]):
        transcoding_container (Union[Unset, None, str]):
        transcoding_protocol (Union[Unset, None, str]):
        max_audio_sample_rate (Union[Unset, None, int]):
        max_audio_bit_depth (Union[Unset, None, int]):
        enable_remote_media (Union[Unset, None, bool]):
        break_on_non_key_frames (Union[Unset, None, bool]):
        enable_redirection (Union[Unset, None, bool]):  Default: True.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        item_id=item_id,
        container=container,
        media_source_id=media_source_id,
        device_id=device_id,
        user_id=user_id,
        audio_codec=audio_codec,
        max_audio_channels=max_audio_channels,
        transcoding_audio_channels=transcoding_audio_channels,
        max_streaming_bitrate=max_streaming_bitrate,
        audio_bit_rate=audio_bit_rate,
        start_time_ticks=start_time_ticks,
        transcoding_container=transcoding_container,
        transcoding_protocol=transcoding_protocol,
        max_audio_sample_rate=max_audio_sample_rate,
        max_audio_bit_depth=max_audio_bit_depth,
        enable_remote_media=enable_remote_media,
        break_on_non_key_frames=break_on_non_key_frames,
        enable_redirection=enable_redirection,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.image_option import ImageOption


T = TypeVar("T", bound="TypeOptions")


@_attrs_define
class TypeOptions:
    """
    Attributes:
        type (Union[Unset, None, str]):
        metadata_fetchers (Union[Unset, None, List[str]]):
        metadata_fetcher_order (Union[Unset, None, List[str]]):
        image_fetchers (Union[Unset, None, List[str]]):
        image_fetcher_order (Union[Unset, None, List[str]]):
        image_options (Union[Unset, None, List['ImageOption']]):
    """

    type: Union[Unset, None, str] = UNSET
    metadata_fetchers: Union[Unset, None, List[str]] = UNSET
    metadata_fetcher_order: Union[Unset, None, List[str]] = UNSET
    image_fetchers: Union[Unset, None, List[str]] = UNSET
    image_fetcher_order: Union[Unset, None, List[str]] = UNSET
    image_options: Union[Unset, None, List["ImageOption"]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type = self.type
        metadata_fetchers: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.metadata_fetchers, Unset):
            if self.metadata_fetchers is None:
                metadata_fetchers = None
            else:
                metadata_fetchers = self.metadata_fetchers

        metadata_fetcher_order: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.metadata_fetcher_order, Unset):
            if self.metadata_fetcher_order is None:
                metadata_fetcher_order = None
            else:
                metadata_fetcher_order = self.metadata_fetcher_order

        image_fetchers: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.image_fetchers, Unset):
            if self.image_fetchers is None:
                image_fetchers = None
            else:
                image_fetchers = self.image_fetchers

        image_fetcher_order: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.image_fetcher_order, Unset):
            if self.image_fetcher_order is None:
                image_fetcher_order = None
            else:
                image_fetcher_order = self.image_fetcher_order

        image_options: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.image_options, Unset):
            if self.image_options is None:
                image_options = None
            else:
                image_options = []
                for image_options_item_data in self.image_options:
                    image_options_item = image_options_item_data.to_dict()

                    image_options.append(image_options_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if type is not UNSET:
            field_dict["Type"] = type
        if metadata_fetchers is not UNSET:
            field_dict["MetadataFetchers"] = metadata_fetchers
        if metadata_fetcher_order is not UNSET:
            field_dict["MetadataFetcherOrder"] = metadata_fetcher_order
        if image_fetchers is not UNSET:
            field_dict["ImageFetchers"] = image_fetchers
        if image_fetcher_order is not UNSET:
            field_dict["ImageFetcherOrder"] = image_fetcher_order
        if image_options is not UNSET:
            field_dict["ImageOptions"] = image_options

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.image_option import ImageOption

        d = src_dict.copy()
        type = d.pop("Type", UNSET)

        metadata_fetchers = cast(List[str], d.pop("MetadataFetchers", UNSET))

        metadata_fetcher_order = cast(List[str], d.pop("MetadataFetcherOrder", UNSET))

        image_fetchers = cast(List[str], d.pop("ImageFetchers", UNSET))

        image_fetcher_order = cast(List[str], d.pop("ImageFetcherOrder", UNSET))

        image_options = []
        _image_options = d.pop("ImageOptions", UNSET)
        for image_options_item_data in _image_options or []:
            image_options_item = ImageOption.from_dict(image_options_item_data)

            image_options.append(image_options_item)

        type_options = cls(
            type=type,
            metadata_fetchers=metadata_fetchers,
            metadata_fetcher_order=metadata_fetcher_order,
            image_fetchers=image_fetchers,
            image_fetcher_order=image_fetcher_order,
            image_options=image_options,
        )

        return type_options



import datetime
from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="RemoteSubtitleInfo")


@_attrs_define
class RemoteSubtitleInfo:
    """
    Attributes:
        three_letter_iso_language_name (Union[Unset, None, str]):
        id (Union[Unset, None, str]):
        provider_name (Union[Unset, None, str]):
        name (Union[Unset, None, str]):
        format_ (Union[Unset, None, str]):
        author (Union[Unset, None, str]):
        comment (Union[Unset, None, str]):
        date_created (Union[Unset, None, datetime.datetime]):
        community_rating (Union[Unset, None, float]):
        download_count (Union[Unset, None, int]):
        is_hash_match (Union[Unset, None, bool]):
    """

    three_letter_iso_language_name: Union[Unset, None, str] = UNSET
    id: Union[Unset, None, str] = UNSET
    provider_name: Union[Unset, None, str] = UNSET
    name: Union[Unset, None, str] = UNSET
    format_: Union[Unset, None, str] = UNSET
    author: Union[Unset, None, str] = UNSET
    comment: Union[Unset, None, str] = UNSET
    date_created: Union[Unset, None, datetime.datetime] = UNSET
    community_rating: Union[Unset, None, float] = UNSET
    download_count: Union[Unset, None, int] = UNSET
    is_hash_match: Union[Unset, None, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        three_letter_iso_language_name = self.three_letter_iso_language_name
        id = self.id
        provider_name = self.provider_name
        name = self.name
        format_ = self.format_
        author = self.author
        comment = self.comment
        date_created: Union[Unset, None, str] = UNSET
        if not isinstance(self.date_created, Unset):
            date_created = self.date_created.isoformat() if self.date_created else None

        community_rating = self.community_rating
        download_count = self.download_count
        is_hash_match = self.is_hash_match

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if three_letter_iso_language_name is not UNSET:
            field_dict["ThreeLetterISOLanguageName"] = three_letter_iso_language_name
        if id is not UNSET:
            field_dict["Id"] = id
        if provider_name is not UNSET:
            field_dict["ProviderName"] = provider_name
        if name is not UNSET:
            field_dict["Name"] = name
        if format_ is not UNSET:
            field_dict["Format"] = format_
        if author is not UNSET:
            field_dict["Author"] = author
        if comment is not UNSET:
            field_dict["Comment"] = comment
        if date_created is not UNSET:
            field_dict["DateCreated"] = date_created
        if community_rating is not UNSET:
            field_dict["CommunityRating"] = community_rating
        if download_count is not UNSET:
            field_dict["DownloadCount"] = download_count
        if is_hash_match is not UNSET:
            field_dict["IsHashMatch"] = is_hash_match

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        three_letter_iso_language_name = d.pop("ThreeLetterISOLanguageName", UNSET)

        id = d.pop("Id", UNSET)

        provider_name = d.pop("ProviderName", UNSET)

        name = d.pop("Name", UNSET)

        format_ = d.pop("Format", UNSET)

        author = d.pop("Author", UNSET)

        comment = d.pop("Comment", UNSET)

        _date_created = d.pop("DateCreated", UNSET)
        date_created: Union[Unset, None, datetime.datetime]
        if _date_created is None:
            date_created = None
        elif isinstance(_date_created, Unset):
            date_created = UNSET
        else:
            date_created = isoparse(_date_created)

        community_rating = d.pop("CommunityRating", UNSET)

        download_count = d.pop("DownloadCount", UNSET)

        is_hash_match = d.pop("IsHashMatch", UNSET)

        remote_subtitle_info = cls(
            three_letter_iso_language_name=three_letter_iso_language_name,
            id=id,
            provider_name=provider_name,
            name=name,
            format_=format_,
            author=author,
            comment=comment,
            date_created=date_created,
            community_rating=community_rating,
            download_count=download_count,
            is_hash_match=is_hash_match,
        )

        return remote_subtitle_info



import datetime
from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="GuideInfo")


@_attrs_define
class GuideInfo:
    """
    Attributes:
        start_date (Union[Unset, datetime.datetime]): Gets or sets the start date.
        end_date (Union[Unset, datetime.datetime]): Gets or sets the end date.
    """

    start_date: Union[Unset, datetime.datetime] = UNSET
    end_date: Union[Unset, datetime.datetime] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if start_date is not UNSET:
            field_dict["StartDate"] = start_date
        if end_date is not UNSET:
            field_dict["EndDate"] = end_date

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _start_date = d.pop("StartDate", UNSET)
        start_date: Union[Unset, datetime.datetime]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date)

        _end_date = d.pop("EndDate", UNSET)
        end_date: Union[Unset, datetime.datetime]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date)

        guide_info = cls(
            start_date=start_date,
            end_date=end_date,
        )

        return guide_info



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.remote_search_result_provider_ids import RemoteSearchResultProviderIds


T = TypeVar("T", bound="RemoteSearchResult")


@_attrs_define
class RemoteSearchResult:
    """
    Attributes:
        name (Union[Unset, None, str]): Gets or sets the name.
        provider_ids (Union[Unset, None, RemoteSearchResultProviderIds]): Gets or sets the provider ids.
        production_year (Union[Unset, None, int]): Gets or sets the year.
        index_number (Union[Unset, None, int]):
        index_number_end (Union[Unset, None, int]):
        parent_index_number (Union[Unset, None, int]):
        premiere_date (Union[Unset, None, datetime.datetime]):
        image_url (Union[Unset, None, str]):
        search_provider_name (Union[Unset, None, str]):
        overview (Union[Unset, None, str]):
        album_artist (Union[Unset, None, RemoteSearchResult]):
        artists (Union[Unset, None, List['RemoteSearchResult']]):
    """

    name: Union[Unset, None, str] = UNSET
    provider_ids: Union[Unset, None, "RemoteSearchResultProviderIds"] = UNSET
    production_year: Union[Unset, None, int] = UNSET
    index_number: Union[Unset, None, int] = UNSET
    index_number_end: Union[Unset, None, int] = UNSET
    parent_index_number: Union[Unset, None, int] = UNSET
    premiere_date: Union[Unset, None, datetime.datetime] = UNSET
    image_url: Union[Unset, None, str] = UNSET
    search_provider_name: Union[Unset, None, str] = UNSET
    overview: Union[Unset, None, str] = UNSET
    album_artist: Union[Unset, None, "RemoteSearchResult"] = UNSET
    artists: Union[Unset, None, List["RemoteSearchResult"]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        provider_ids: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.provider_ids, Unset):
            provider_ids = self.provider_ids.to_dict() if self.provider_ids else None

        production_year = self.production_year
        index_number = self.index_number
        index_number_end = self.index_number_end
        parent_index_number = self.parent_index_number
        premiere_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.premiere_date, Unset):
            premiere_date = self.premiere_date.isoformat() if self.premiere_date else None

        image_url = self.image_url
        search_provider_name = self.search_provider_name
        overview = self.overview
        album_artist: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.album_artist, Unset):
            album_artist = self.album_artist.to_dict() if self.album_artist else None

        artists: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.artists, Unset):
            if self.artists is None:
                artists = None
            else:
                artists = []
                for artists_item_data in self.artists:
                    artists_item = artists_item_data.to_dict()

                    artists.append(artists_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["Name"] = name
        if provider_ids is not UNSET:
            field_dict["ProviderIds"] = provider_ids
        if production_year is not UNSET:
            field_dict["ProductionYear"] = production_year
        if index_number is not UNSET:
            field_dict["IndexNumber"] = index_number
        if index_number_end is not UNSET:
            field_dict["IndexNumberEnd"] = index_number_end
        if parent_index_number is not UNSET:
            field_dict["ParentIndexNumber"] = parent_index_number
        if premiere_date is not UNSET:
            field_dict["PremiereDate"] = premiere_date
        if image_url is not UNSET:
            field_dict["ImageUrl"] = image_url
        if search_provider_name is not UNSET:
            field_dict["SearchProviderName"] = search_provider_name
        if overview is not UNSET:
            field_dict["Overview"] = overview
        if album_artist is not UNSET:
            field_dict["AlbumArtist"] = album_artist
        if artists is not UNSET:
            field_dict["Artists"] = artists

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.remote_search_result_provider_ids import RemoteSearchResultProviderIds

        d = src_dict.copy()
        name = d.pop("Name", UNSET)

        _provider_ids = d.pop("ProviderIds", UNSET)
        provider_ids: Union[Unset, None, RemoteSearchResultProviderIds]
        if _provider_ids is None:
            provider_ids = None
        elif isinstance(_provider_ids, Unset):
            provider_ids = UNSET
        else:
            provider_ids = RemoteSearchResultProviderIds.from_dict(_provider_ids)

        production_year = d.pop("ProductionYear", UNSET)

        index_number = d.pop("IndexNumber", UNSET)

        index_number_end = d.pop("IndexNumberEnd", UNSET)

        parent_index_number = d.pop("ParentIndexNumber", UNSET)

        _premiere_date = d.pop("PremiereDate", UNSET)
        premiere_date: Union[Unset, None, datetime.datetime]
        if _premiere_date is None:
            premiere_date = None
        elif isinstance(_premiere_date, Unset):
            premiere_date = UNSET
        else:
            premiere_date = isoparse(_premiere_date)

        image_url = d.pop("ImageUrl", UNSET)

        search_provider_name = d.pop("SearchProviderName", UNSET)

        overview = d.pop("Overview", UNSET)

        _album_artist = d.pop("AlbumArtist", UNSET)
        album_artist: Union[Unset, None, RemoteSearchResult]
        if _album_artist is None:
            album_artist = None
        elif isinstance(_album_artist, Unset):
            album_artist = UNSET
        else:
            album_artist = RemoteSearchResult.from_dict(_album_artist)

        artists = []
        _artists = d.pop("Artists", UNSET)
        for artists_item_data in _artists or []:
            artists_item = RemoteSearchResult.from_dict(artists_item_data)

            artists.append(artists_item)

        remote_search_result = cls(
            name=name,
            provider_ids=provider_ids,
            production_year=production_year,
            index_number=index_number,
            index_number_end=index_number_end,
            parent_index_number=parent_index_number,
            premiere_date=premiere_date,
            image_url=image_url,
            search_provider_name=search_provider_name,
            overview=overview,
            album_artist=album_artist,
            artists=artists,
        )

        return remote_search_result



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.control_response_headers import ControlResponseHeaders


T = TypeVar("T", bound="ControlResponse")


@_attrs_define
class ControlResponse:
    """
    Attributes:
        headers (Union[Unset, ControlResponseHeaders]):
        xml (Union[Unset, str]):
        is_successful (Union[Unset, bool]):
    """

    headers: Union[Unset, "ControlResponseHeaders"] = UNSET
    xml: Union[Unset, str] = UNSET
    is_successful: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        headers: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.headers, Unset):
            headers = self.headers.to_dict()

        xml = self.xml
        is_successful = self.is_successful

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if headers is not UNSET:
            field_dict["Headers"] = headers
        if xml is not UNSET:
            field_dict["Xml"] = xml
        if is_successful is not UNSET:
            field_dict["IsSuccessful"] = is_successful

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.control_response_headers import ControlResponseHeaders

        d = src_dict.copy()
        _headers = d.pop("Headers", UNSET)
        headers: Union[Unset, ControlResponseHeaders]
        if isinstance(_headers, Unset):
            headers = UNSET
        else:
            headers = ControlResponseHeaders.from_dict(_headers)

        xml = d.pop("Xml", UNSET)

        is_successful = d.pop("IsSuccessful", UNSET)

        control_response = cls(
            headers=headers,
            xml=xml,
            is_successful=is_successful,
        )

        return control_response



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.base_item_person_image_blur_hashes_art import BaseItemPersonImageBlurHashesArt
    from ..models.base_item_person_image_blur_hashes_backdrop import BaseItemPersonImageBlurHashesBackdrop
    from ..models.base_item_person_image_blur_hashes_banner import BaseItemPersonImageBlurHashesBanner
    from ..models.base_item_person_image_blur_hashes_box import BaseItemPersonImageBlurHashesBox
    from ..models.base_item_person_image_blur_hashes_box_rear import BaseItemPersonImageBlurHashesBoxRear
    from ..models.base_item_person_image_blur_hashes_chapter import BaseItemPersonImageBlurHashesChapter
    from ..models.base_item_person_image_blur_hashes_disc import BaseItemPersonImageBlurHashesDisc
    from ..models.base_item_person_image_blur_hashes_logo import BaseItemPersonImageBlurHashesLogo
    from ..models.base_item_person_image_blur_hashes_menu import BaseItemPersonImageBlurHashesMenu
    from ..models.base_item_person_image_blur_hashes_primary import BaseItemPersonImageBlurHashesPrimary
    from ..models.base_item_person_image_blur_hashes_profile import BaseItemPersonImageBlurHashesProfile
    from ..models.base_item_person_image_blur_hashes_screenshot import BaseItemPersonImageBlurHashesScreenshot
    from ..models.base_item_person_image_blur_hashes_thumb import BaseItemPersonImageBlurHashesThumb


T = TypeVar("T", bound="BaseItemPersonImageBlurHashes")


@_attrs_define
class BaseItemPersonImageBlurHashes:
    """Gets or sets the primary image blurhash.

    Attributes:
        primary (Union[Unset, BaseItemPersonImageBlurHashesPrimary]):
        art (Union[Unset, BaseItemPersonImageBlurHashesArt]):
        backdrop (Union[Unset, BaseItemPersonImageBlurHashesBackdrop]):
        banner (Union[Unset, BaseItemPersonImageBlurHashesBanner]):
        logo (Union[Unset, BaseItemPersonImageBlurHashesLogo]):
        thumb (Union[Unset, BaseItemPersonImageBlurHashesThumb]):
        disc (Union[Unset, BaseItemPersonImageBlurHashesDisc]):
        box (Union[Unset, BaseItemPersonImageBlurHashesBox]):
        screenshot (Union[Unset, BaseItemPersonImageBlurHashesScreenshot]):
        menu (Union[Unset, BaseItemPersonImageBlurHashesMenu]):
        chapter (Union[Unset, BaseItemPersonImageBlurHashesChapter]):
        box_rear (Union[Unset, BaseItemPersonImageBlurHashesBoxRear]):
        profile (Union[Unset, BaseItemPersonImageBlurHashesProfile]):
    """

    primary: Union[Unset, "BaseItemPersonImageBlurHashesPrimary"] = UNSET
    art: Union[Unset, "BaseItemPersonImageBlurHashesArt"] = UNSET
    backdrop: Union[Unset, "BaseItemPersonImageBlurHashesBackdrop"] = UNSET
    banner: Union[Unset, "BaseItemPersonImageBlurHashesBanner"] = UNSET
    logo: Union[Unset, "BaseItemPersonImageBlurHashesLogo"] = UNSET
    thumb: Union[Unset, "BaseItemPersonImageBlurHashesThumb"] = UNSET
    disc: Union[Unset, "BaseItemPersonImageBlurHashesDisc"] = UNSET
    box: Union[Unset, "BaseItemPersonImageBlurHashesBox"] = UNSET
    screenshot: Union[Unset, "BaseItemPersonImageBlurHashesScreenshot"] = UNSET
    menu: Union[Unset, "BaseItemPersonImageBlurHashesMenu"] = UNSET
    chapter: Union[Unset, "BaseItemPersonImageBlurHashesChapter"] = UNSET
    box_rear: Union[Unset, "BaseItemPersonImageBlurHashesBoxRear"] = UNSET
    profile: Union[Unset, "BaseItemPersonImageBlurHashesProfile"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        primary: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.primary, Unset):
            primary = self.primary.to_dict()

        art: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.art, Unset):
            art = self.art.to_dict()

        backdrop: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.backdrop, Unset):
            backdrop = self.backdrop.to_dict()

        banner: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.banner, Unset):
            banner = self.banner.to_dict()

        logo: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.logo, Unset):
            logo = self.logo.to_dict()

        thumb: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.thumb, Unset):
            thumb = self.thumb.to_dict()

        disc: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.disc, Unset):
            disc = self.disc.to_dict()

        box: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.box, Unset):
            box = self.box.to_dict()

        screenshot: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.screenshot, Unset):
            screenshot = self.screenshot.to_dict()

        menu: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.menu, Unset):
            menu = self.menu.to_dict()

        chapter: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.chapter, Unset):
            chapter = self.chapter.to_dict()

        box_rear: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.box_rear, Unset):
            box_rear = self.box_rear.to_dict()

        profile: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile, Unset):
            profile = self.profile.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if primary is not UNSET:
            field_dict["Primary"] = primary
        if art is not UNSET:
            field_dict["Art"] = art
        if backdrop is not UNSET:
            field_dict["Backdrop"] = backdrop
        if banner is not UNSET:
            field_dict["Banner"] = banner
        if logo is not UNSET:
            field_dict["Logo"] = logo
        if thumb is not UNSET:
            field_dict["Thumb"] = thumb
        if disc is not UNSET:
            field_dict["Disc"] = disc
        if box is not UNSET:
            field_dict["Box"] = box
        if screenshot is not UNSET:
            field_dict["Screenshot"] = screenshot
        if menu is not UNSET:
            field_dict["Menu"] = menu
        if chapter is not UNSET:
            field_dict["Chapter"] = chapter
        if box_rear is not UNSET:
            field_dict["BoxRear"] = box_rear
        if profile is not UNSET:
            field_dict["Profile"] = profile

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.base_item_person_image_blur_hashes_art import BaseItemPersonImageBlurHashesArt
        from ..models.base_item_person_image_blur_hashes_backdrop import BaseItemPersonImageBlurHashesBackdrop
        from ..models.base_item_person_image_blur_hashes_banner import BaseItemPersonImageBlurHashesBanner
        from ..models.base_item_person_image_blur_hashes_box import BaseItemPersonImageBlurHashesBox
        from ..models.base_item_person_image_blur_hashes_box_rear import BaseItemPersonImageBlurHashesBoxRear
        from ..models.base_item_person_image_blur_hashes_chapter import BaseItemPersonImageBlurHashesChapter
        from ..models.base_item_person_image_blur_hashes_disc import BaseItemPersonImageBlurHashesDisc
        from ..models.base_item_person_image_blur_hashes_logo import BaseItemPersonImageBlurHashesLogo
        from ..models.base_item_person_image_blur_hashes_menu import BaseItemPersonImageBlurHashesMenu
        from ..models.base_item_person_image_blur_hashes_primary import BaseItemPersonImageBlurHashesPrimary
        from ..models.base_item_person_image_blur_hashes_profile import BaseItemPersonImageBlurHashesProfile
        from ..models.base_item_person_image_blur_hashes_screenshot import BaseItemPersonImageBlurHashesScreenshot
        from ..models.base_item_person_image_blur_hashes_thumb import BaseItemPersonImageBlurHashesThumb

        d = src_dict.copy()
        _primary = d.pop("Primary", UNSET)
        primary: Union[Unset, BaseItemPersonImageBlurHashesPrimary]
        if isinstance(_primary, Unset):
            primary = UNSET
        else:
            primary = BaseItemPersonImageBlurHashesPrimary.from_dict(_primary)

        _art = d.pop("Art", UNSET)
        art: Union[Unset, BaseItemPersonImageBlurHashesArt]
        if isinstance(_art, Unset):
            art = UNSET
        else:
            art = BaseItemPersonImageBlurHashesArt.from_dict(_art)

        _backdrop = d.pop("Backdrop", UNSET)
        backdrop: Union[Unset, BaseItemPersonImageBlurHashesBackdrop]
        if isinstance(_backdrop, Unset):
            backdrop = UNSET
        else:
            backdrop = BaseItemPersonImageBlurHashesBackdrop.from_dict(_backdrop)

        _banner = d.pop("Banner", UNSET)
        banner: Union[Unset, BaseItemPersonImageBlurHashesBanner]
        if isinstance(_banner, Unset):
            banner = UNSET
        else:
            banner = BaseItemPersonImageBlurHashesBanner.from_dict(_banner)

        _logo = d.pop("Logo", UNSET)
        logo: Union[Unset, BaseItemPersonImageBlurHashesLogo]
        if isinstance(_logo, Unset):
            logo = UNSET
        else:
            logo = BaseItemPersonImageBlurHashesLogo.from_dict(_logo)

        _thumb = d.pop("Thumb", UNSET)
        thumb: Union[Unset, BaseItemPersonImageBlurHashesThumb]
        if isinstance(_thumb, Unset):
            thumb = UNSET
        else:
            thumb = BaseItemPersonImageBlurHashesThumb.from_dict(_thumb)

        _disc = d.pop("Disc", UNSET)
        disc: Union[Unset, BaseItemPersonImageBlurHashesDisc]
        if isinstance(_disc, Unset):
            disc = UNSET
        else:
            disc = BaseItemPersonImageBlurHashesDisc.from_dict(_disc)

        _box = d.pop("Box", UNSET)
        box: Union[Unset, BaseItemPersonImageBlurHashesBox]
        if isinstance(_box, Unset):
            box = UNSET
        else:
            box = BaseItemPersonImageBlurHashesBox.from_dict(_box)

        _screenshot = d.pop("Screenshot", UNSET)
        screenshot: Union[Unset, BaseItemPersonImageBlurHashesScreenshot]
        if isinstance(_screenshot, Unset):
            screenshot = UNSET
        else:
            screenshot = BaseItemPersonImageBlurHashesScreenshot.from_dict(_screenshot)

        _menu = d.pop("Menu", UNSET)
        menu: Union[Unset, BaseItemPersonImageBlurHashesMenu]
        if isinstance(_menu, Unset):
            menu = UNSET
        else:
            menu = BaseItemPersonImageBlurHashesMenu.from_dict(_menu)

        _chapter = d.pop("Chapter", UNSET)
        chapter: Union[Unset, BaseItemPersonImageBlurHashesChapter]
        if isinstance(_chapter, Unset):
            chapter = UNSET
        else:
            chapter = BaseItemPersonImageBlurHashesChapter.from_dict(_chapter)

        _box_rear = d.pop("BoxRear", UNSET)
        box_rear: Union[Unset, BaseItemPersonImageBlurHashesBoxRear]
        if isinstance(_box_rear, Unset):
            box_rear = UNSET
        else:
            box_rear = BaseItemPersonImageBlurHashesBoxRear.from_dict(_box_rear)

        _profile = d.pop("Profile", UNSET)
        profile: Union[Unset, BaseItemPersonImageBlurHashesProfile]
        if isinstance(_profile, Unset):
            profile = UNSET
        else:
            profile = BaseItemPersonImageBlurHashesProfile.from_dict(_profile)

        base_item_person_image_blur_hashes = cls(
            primary=primary,
            art=art,
            backdrop=backdrop,
            banner=banner,
            logo=logo,
            thumb=thumb,
            disc=disc,
            box=box,
            screenshot=screenshot,
            menu=menu,
            chapter=chapter,
            box_rear=box_rear,
            profile=profile,
        )

        base_item_person_image_blur_hashes.additional_properties = d
        return base_item_person_image_blur_hashes

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="MetadataConfiguration")


@_attrs_define
class MetadataConfiguration:
    """
    Attributes:
        use_file_creation_time_for_date_added (Union[Unset, bool]):
    """

    use_file_creation_time_for_date_added: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        use_file_creation_time_for_date_added = self.use_file_creation_time_for_date_added

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if use_file_creation_time_for_date_added is not UNSET:
            field_dict["UseFileCreationTimeForDateAdded"] = use_file_creation_time_for_date_added

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        use_file_creation_time_for_date_added = d.pop("UseFileCreationTimeForDateAdded", UNSET)

        metadata_configuration = cls(
            use_file_creation_time_for_date_added=use_file_creation_time_for_date_added,
        )

        return metadata_configuration



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.image_format import ImageFormat
from ...models.image_type import ImageType
from ...models.problem_details import ProblemDetails
from ...types import UNSET, Response, Unset


def _get_kwargs(
    name: str,
    image_type: ImageType,
    *,
    tag: Union[Unset, None, str] = UNSET,
    format_: Union[Unset, None, ImageFormat] = UNSET,
    max_width: Union[Unset, None, int] = UNSET,
    max_height: Union[Unset, None, int] = UNSET,
    percent_played: Union[Unset, None, float] = UNSET,
    unplayed_count: Union[Unset, None, int] = UNSET,
    width: Union[Unset, None, int] = UNSET,
    height: Union[Unset, None, int] = UNSET,
    quality: Union[Unset, None, int] = UNSET,
    fill_width: Union[Unset, None, int] = UNSET,
    fill_height: Union[Unset, None, int] = UNSET,
    crop_whitespace: Union[Unset, None, bool] = UNSET,
    add_played_indicator: Union[Unset, None, bool] = UNSET,
    blur: Union[Unset, None, int] = UNSET,
    background_color: Union[Unset, None, str] = UNSET,
    foreground_layer: Union[Unset, None, str] = UNSET,
    image_index: Union[Unset, None, int] = UNSET,
) -> Dict[str, Any]:
    pass

    params: Dict[str, Any] = {}
    params["tag"] = tag

    json_format_: Union[Unset, None, str] = UNSET
    if not isinstance(format_, Unset):
        json_format_ = format_.value if format_ else None

    params["format"] = json_format_

    params["maxWidth"] = max_width

    params["maxHeight"] = max_height

    params["percentPlayed"] = percent_played

    params["unplayedCount"] = unplayed_count

    params["width"] = width

    params["height"] = height

    params["quality"] = quality

    params["fillWidth"] = fill_width

    params["fillHeight"] = fill_height

    params["cropWhitespace"] = crop_whitespace

    params["addPlayedIndicator"] = add_played_indicator

    params["blur"] = blur

    params["backgroundColor"] = background_color

    params["foregroundLayer"] = foreground_layer

    params["imageIndex"] = image_index

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/Studios/{name}/Images/{imageType}".format(
            name=name,
            imageType=image_type,
        ),
        "params": params,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[ProblemDetails]:
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = ProblemDetails.from_dict(response.json())

        return response_404
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[ProblemDetails]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    name: str,
    image_type: ImageType,
    *,
    client: Union[AuthenticatedClient, Client],
    tag: Union[Unset, None, str] = UNSET,
    format_: Union[Unset, None, ImageFormat] = UNSET,
    max_width: Union[Unset, None, int] = UNSET,
    max_height: Union[Unset, None, int] = UNSET,
    percent_played: Union[Unset, None, float] = UNSET,
    unplayed_count: Union[Unset, None, int] = UNSET,
    width: Union[Unset, None, int] = UNSET,
    height: Union[Unset, None, int] = UNSET,
    quality: Union[Unset, None, int] = UNSET,
    fill_width: Union[Unset, None, int] = UNSET,
    fill_height: Union[Unset, None, int] = UNSET,
    crop_whitespace: Union[Unset, None, bool] = UNSET,
    add_played_indicator: Union[Unset, None, bool] = UNSET,
    blur: Union[Unset, None, int] = UNSET,
    background_color: Union[Unset, None, str] = UNSET,
    foreground_layer: Union[Unset, None, str] = UNSET,
    image_index: Union[Unset, None, int] = UNSET,
) -> Response[ProblemDetails]:
    """Get studio image by name.

    Args:
        name (str):
        image_type (ImageType): Enum ImageType.
        tag (Union[Unset, None, str]):
        format_ (Union[Unset, None, ImageFormat]): Enum ImageOutputFormat.
        max_width (Union[Unset, None, int]):
        max_height (Union[Unset, None, int]):
        percent_played (Union[Unset, None, float]):
        unplayed_count (Union[Unset, None, int]):
        width (Union[Unset, None, int]):
        height (Union[Unset, None, int]):
        quality (Union[Unset, None, int]):
        fill_width (Union[Unset, None, int]):
        fill_height (Union[Unset, None, int]):
        crop_whitespace (Union[Unset, None, bool]):
        add_played_indicator (Union[Unset, None, bool]):
        blur (Union[Unset, None, int]):
        background_color (Union[Unset, None, str]):
        foreground_layer (Union[Unset, None, str]):
        image_index (Union[Unset, None, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ProblemDetails]
    """

    kwargs = _get_kwargs(
        name=name,
        image_type=image_type,
        tag=tag,
        format_=format_,
        max_width=max_width,
        max_height=max_height,
        percent_played=percent_played,
        unplayed_count=unplayed_count,
        width=width,
        height=height,
        quality=quality,
        fill_width=fill_width,
        fill_height=fill_height,
        crop_whitespace=crop_whitespace,
        add_played_indicator=add_played_indicator,
        blur=blur,
        background_color=background_color,
        foreground_layer=foreground_layer,
        image_index=image_index,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    name: str,
    image_type: ImageType,
    *,
    client: Union[AuthenticatedClient, Client],
    tag: Union[Unset, None, str] = UNSET,
    format_: Union[Unset, None, ImageFormat] = UNSET,
    max_width: Union[Unset, None, int] = UNSET,
    max_height: Union[Unset, None, int] = UNSET,
    percent_played: Union[Unset, None, float] = UNSET,
    unplayed_count: Union[Unset, None, int] = UNSET,
    width: Union[Unset, None, int] = UNSET,
    height: Union[Unset, None, int] = UNSET,
    quality: Union[Unset, None, int] = UNSET,
    fill_width: Union[Unset, None, int] = UNSET,
    fill_height: Union[Unset, None, int] = UNSET,
    crop_whitespace: Union[Unset, None, bool] = UNSET,
    add_played_indicator: Union[Unset, None, bool] = UNSET,
    blur: Union[Unset, None, int] = UNSET,
    background_color: Union[Unset, None, str] = UNSET,
    foreground_layer: Union[Unset, None, str] = UNSET,
    image_index: Union[Unset, None, int] = UNSET,
) -> Optional[ProblemDetails]:
    """Get studio image by name.

    Args:
        name (str):
        image_type (ImageType): Enum ImageType.
        tag (Union[Unset, None, str]):
        format_ (Union[Unset, None, ImageFormat]): Enum ImageOutputFormat.
        max_width (Union[Unset, None, int]):
        max_height (Union[Unset, None, int]):
        percent_played (Union[Unset, None, float]):
        unplayed_count (Union[Unset, None, int]):
        width (Union[Unset, None, int]):
        height (Union[Unset, None, int]):
        quality (Union[Unset, None, int]):
        fill_width (Union[Unset, None, int]):
        fill_height (Union[Unset, None, int]):
        crop_whitespace (Union[Unset, None, bool]):
        add_played_indicator (Union[Unset, None, bool]):
        blur (Union[Unset, None, int]):
        background_color (Union[Unset, None, str]):
        foreground_layer (Union[Unset, None, str]):
        image_index (Union[Unset, None, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ProblemDetails
    """

    return sync_detailed(
        name=name,
        image_type=image_type,
        client=client,
        tag=tag,
        format_=format_,
        max_width=max_width,
        max_height=max_height,
        percent_played=percent_played,
        unplayed_count=unplayed_count,
        width=width,
        height=height,
        quality=quality,
        fill_width=fill_width,
        fill_height=fill_height,
        crop_whitespace=crop_whitespace,
        add_played_indicator=add_played_indicator,
        blur=blur,
        background_color=background_color,
        foreground_layer=foreground_layer,
        image_index=image_index,
    ).parsed


async def asyncio_detailed(
    name: str,
    image_type: ImageType,
    *,
    client: Union[AuthenticatedClient, Client],
    tag: Union[Unset, None, str] = UNSET,
    format_: Union[Unset, None, ImageFormat] = UNSET,
    max_width: Union[Unset, None, int] = UNSET,
    max_height: Union[Unset, None, int] = UNSET,
    percent_played: Union[Unset, None, float] = UNSET,
    unplayed_count: Union[Unset, None, int] = UNSET,
    width: Union[Unset, None, int] = UNSET,
    height: Union[Unset, None, int] = UNSET,
    quality: Union[Unset, None, int] = UNSET,
    fill_width: Union[Unset, None, int] = UNSET,
    fill_height: Union[Unset, None, int] = UNSET,
    crop_whitespace: Union[Unset, None, bool] = UNSET,
    add_played_indicator: Union[Unset, None, bool] = UNSET,
    blur: Union[Unset, None, int] = UNSET,
    background_color: Union[Unset, None, str] = UNSET,
    foreground_layer: Union[Unset, None, str] = UNSET,
    image_index: Union[Unset, None, int] = UNSET,
) -> Response[ProblemDetails]:
    """Get studio image by name.

    Args:
        name (str):
        image_type (ImageType): Enum ImageType.
        tag (Union[Unset, None, str]):
        format_ (Union[Unset, None, ImageFormat]): Enum ImageOutputFormat.
        max_width (Union[Unset, None, int]):
        max_height (Union[Unset, None, int]):
        percent_played (Union[Unset, None, float]):
        unplayed_count (Union[Unset, None, int]):
        width (Union[Unset, None, int]):
        height (Union[Unset, None, int]):
        quality (Union[Unset, None, int]):
        fill_width (Union[Unset, None, int]):
        fill_height (Union[Unset, None, int]):
        crop_whitespace (Union[Unset, None, bool]):
        add_played_indicator (Union[Unset, None, bool]):
        blur (Union[Unset, None, int]):
        background_color (Union[Unset, None, str]):
        foreground_layer (Union[Unset, None, str]):
        image_index (Union[Unset, None, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ProblemDetails]
    """

    kwargs = _get_kwargs(
        name=name,
        image_type=image_type,
        tag=tag,
        format_=format_,
        max_width=max_width,
        max_height=max_height,
        percent_played=percent_played,
        unplayed_count=unplayed_count,
        width=width,
        height=height,
        quality=quality,
        fill_width=fill_width,
        fill_height=fill_height,
        crop_whitespace=crop_whitespace,
        add_played_indicator=add_played_indicator,
        blur=blur,
        background_color=background_color,
        foreground_layer=foreground_layer,
        image_index=image_index,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    name: str,
    image_type: ImageType,
    *,
    client: Union[AuthenticatedClient, Client],
    tag: Union[Unset, None, str] = UNSET,
    format_: Union[Unset, None, ImageFormat] = UNSET,
    max_width: Union[Unset, None, int] = UNSET,
    max_height: Union[Unset, None, int] = UNSET,
    percent_played: Union[Unset, None, float] = UNSET,
    unplayed_count: Union[Unset, None, int] = UNSET,
    width: Union[Unset, None, int] = UNSET,
    height: Union[Unset, None, int] = UNSET,
    quality: Union[Unset, None, int] = UNSET,
    fill_width: Union[Unset, None, int] = UNSET,
    fill_height: Union[Unset, None, int] = UNSET,
    crop_whitespace: Union[Unset, None, bool] = UNSET,
    add_played_indicator: Union[Unset, None, bool] = UNSET,
    blur: Union[Unset, None, int] = UNSET,
    background_color: Union[Unset, None, str] = UNSET,
    foreground_layer: Union[Unset, None, str] = UNSET,
    image_index: Union[Unset, None, int] = UNSET,
) -> Optional[ProblemDetails]:
    """Get studio image by name.

    Args:
        name (str):
        image_type (ImageType): Enum ImageType.
        tag (Union[Unset, None, str]):
        format_ (Union[Unset, None, ImageFormat]): Enum ImageOutputFormat.
        max_width (Union[Unset, None, int]):
        max_height (Union[Unset, None, int]):
        percent_played (Union[Unset, None, float]):
        unplayed_count (Union[Unset, None, int]):
        width (Union[Unset, None, int]):
        height (Union[Unset, None, int]):
        quality (Union[Unset, None, int]):
        fill_width (Union[Unset, None, int]):
        fill_height (Union[Unset, None, int]):
        crop_whitespace (Union[Unset, None, bool]):
        add_played_indicator (Union[Unset, None, bool]):
        blur (Union[Unset, None, int]):
        background_color (Union[Unset, None, str]):
        foreground_layer (Union[Unset, None, str]):
        image_index (Union[Unset, None, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ProblemDetails
    """

    return (
        await asyncio_detailed(
            name=name,
            image_type=image_type,
            client=client,
            tag=tag,
            format_=format_,
            max_width=max_width,
            max_height=max_height,
            percent_played=percent_played,
            unplayed_count=unplayed_count,
            width=width,
            height=height,
            quality=quality,
            fill_width=fill_width,
            fill_height=fill_height,
            crop_whitespace=crop_whitespace,
            add_played_indicator=add_played_indicator,
            blur=blur,
            background_color=background_color,
            foreground_layer=foreground_layer,
            image_index=image_index,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..models.notification_level import NotificationLevel
from ..types import UNSET, Unset

T = TypeVar("T", bound="AdminNotificationDto")


@_attrs_define
class AdminNotificationDto:
    """The admin notification dto.

    Attributes:
        name (Union[Unset, None, str]): Gets or sets the notification name.
        description (Union[Unset, None, str]): Gets or sets the notification description.
        notification_level (Union[Unset, None, NotificationLevel]):
        url (Union[Unset, None, str]): Gets or sets the notification url.
    """

    name: Union[Unset, None, str] = UNSET
    description: Union[Unset, None, str] = UNSET
    notification_level: Union[Unset, None, NotificationLevel] = UNSET
    url: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        description = self.description
        notification_level: Union[Unset, None, str] = UNSET
        if not isinstance(self.notification_level, Unset):
            notification_level = self.notification_level.value if self.notification_level else None

        url = self.url

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["Name"] = name
        if description is not UNSET:
            field_dict["Description"] = description
        if notification_level is not UNSET:
            field_dict["NotificationLevel"] = notification_level
        if url is not UNSET:
            field_dict["Url"] = url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("Name", UNSET)

        description = d.pop("Description", UNSET)

        _notification_level = d.pop("NotificationLevel", UNSET)
        notification_level: Union[Unset, None, NotificationLevel]
        if _notification_level is None:
            notification_level = None
        elif isinstance(_notification_level, Unset):
            notification_level = UNSET
        else:
            notification_level = NotificationLevel(_notification_level)

        url = d.pop("Url", UNSET)

        admin_notification_dto = cls(
            name=name,
            description=description,
            notification_level=notification_level,
            url=url,
        )

        return admin_notification_dto



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..models.external_id_media_type import ExternalIdMediaType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExternalIdInfo")


@_attrs_define
class ExternalIdInfo:
    """Represents the external id information for serialization to the client.

    Attributes:
        name (Union[Unset, str]): Gets or sets the display name of the external id provider (IE: IMDB, MusicBrainz,
            etc).
        key (Union[Unset, str]): Gets or sets the unique key for this id. This key should be unique across all
            providers.
        type (Union[Unset, None, ExternalIdMediaType]): The specific media type of an
            MediaBrowser.Model.Providers.ExternalIdInfo.
        url_format_string (Union[Unset, None, str]): Gets or sets the URL format string.
    """

    name: Union[Unset, str] = UNSET
    key: Union[Unset, str] = UNSET
    type: Union[Unset, None, ExternalIdMediaType] = UNSET
    url_format_string: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        key = self.key
        type: Union[Unset, None, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value if self.type else None

        url_format_string = self.url_format_string

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["Name"] = name
        if key is not UNSET:
            field_dict["Key"] = key
        if type is not UNSET:
            field_dict["Type"] = type
        if url_format_string is not UNSET:
            field_dict["UrlFormatString"] = url_format_string

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("Name", UNSET)

        key = d.pop("Key", UNSET)

        _type = d.pop("Type", UNSET)
        type: Union[Unset, None, ExternalIdMediaType]
        if _type is None:
            type = None
        elif isinstance(_type, Unset):
            type = UNSET
        else:
            type = ExternalIdMediaType(_type)

        url_format_string = d.pop("UrlFormatString", UNSET)

        external_id_info = cls(
            name=name,
            key=key,
            type=type,
            url_format_string=url_format_string,
        )

        return external_id_info



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..models.image_saving_convention import ImageSavingConvention
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.metadata_options import MetadataOptions
    from ..models.name_value_pair import NameValuePair
    from ..models.path_substitution import PathSubstitution
    from ..models.repository_info import RepositoryInfo


T = TypeVar("T", bound="ServerConfiguration")


@_attrs_define
class ServerConfiguration:
    """Represents the server configuration.

    Attributes:
        log_file_retention_days (Union[Unset, int]): Gets or sets the number of days we should retain log files.
        is_startup_wizard_completed (Union[Unset, bool]): Gets or sets a value indicating whether this instance is first
            run.
        cache_path (Union[Unset, None, str]): Gets or sets the cache path.
        previous_version (Union[Unset, None, str]): Gets or sets the last known version that was ran using the
            configuration.
        previous_version_str (Union[Unset, None, str]): Gets or sets the stringified PreviousVersion to be
            stored/loaded,
            because System.Version itself isn't xml-serializable.
        enable_metrics (Union[Unset, bool]): Gets or sets a value indicating whether to enable prometheus metrics
            exporting.
        enable_normalized_item_by_name_ids (Union[Unset, bool]):
        is_port_authorized (Union[Unset, bool]): Gets or sets a value indicating whether this instance is port
            authorized.
        quick_connect_available (Union[Unset, bool]): Gets or sets a value indicating whether quick connect is available
            for use on this server.
        enable_case_sensitive_item_ids (Union[Unset, bool]): Gets or sets a value indicating whether [enable case
            sensitive item ids].
        disable_live_tv_channel_user_data_name (Union[Unset, bool]):
        metadata_path (Union[Unset, str]): Gets or sets the metadata path.
        metadata_network_path (Union[Unset, str]):
        preferred_metadata_language (Union[Unset, str]): Gets or sets the preferred metadata language.
        metadata_country_code (Union[Unset, str]): Gets or sets the metadata country code.
        sort_replace_characters (Union[Unset, List[str]]): Gets or sets characters to be replaced with a ' ' in strings
            to create a sort name.
        sort_remove_characters (Union[Unset, List[str]]): Gets or sets characters to be removed from strings to create a
            sort name.
        sort_remove_words (Union[Unset, List[str]]): Gets or sets words to be removed from strings to create a sort
            name.
        min_resume_pct (Union[Unset, int]): Gets or sets the minimum percentage of an item that must be played in order
            for playstate to be updated.
        max_resume_pct (Union[Unset, int]): Gets or sets the maximum percentage of an item that can be played while
            still saving playstate. If this percentage is crossed playstate will be reset to the beginning and the item will
            be marked watched.
        min_resume_duration_seconds (Union[Unset, int]): Gets or sets the minimum duration that an item must have in
            order to be eligible for playstate updates..
        min_audiobook_resume (Union[Unset, int]): Gets or sets the minimum minutes of a book that must be played in
            order for playstate to be updated.
        max_audiobook_resume (Union[Unset, int]): Gets or sets the remaining minutes of a book that can be played while
            still saving playstate. If this percentage is crossed playstate will be reset to the beginning and the item will
            be marked watched.
        library_monitor_delay (Union[Unset, int]): Gets or sets the delay in seconds that we will wait after a file
            system change to try and discover what has been added/removed
            Some delay is necessary with some items because their creation is not atomic.  It involves the creation of
            several
            different directories and files.
        image_saving_convention (Union[Unset, ImageSavingConvention]):
        metadata_options (Union[Unset, List['MetadataOptions']]):
        skip_deserialization_for_basic_types (Union[Unset, bool]):
        server_name (Union[Unset, str]):
        ui_culture (Union[Unset, str]):
        save_metadata_hidden (Union[Unset, bool]):
        content_types (Union[Unset, List['NameValuePair']]):
        remote_client_bitrate_limit (Union[Unset, int]):
        enable_folder_view (Union[Unset, bool]):
        enable_grouping_into_collections (Union[Unset, bool]):
        display_specials_within_seasons (Union[Unset, bool]):
        codecs_used (Union[Unset, List[str]]):
        plugin_repositories (Union[Unset, List['RepositoryInfo']]):
        enable_external_content_in_suggestions (Union[Unset, bool]):
        image_extraction_timeout_ms (Union[Unset, int]):
        path_substitutions (Union[Unset, List['PathSubstitution']]):
        enable_slow_response_warning (Union[Unset, bool]): Gets or sets a value indicating whether slow server responses
            should be logged as a warning.
        slow_response_threshold_ms (Union[Unset, int]): Gets or sets the threshold for the slow response time warning in
            ms.
        cors_hosts (Union[Unset, List[str]]): Gets or sets the cors hosts.
        activity_log_retention_days (Union[Unset, None, int]): Gets or sets the number of days we should retain activity
            logs.
        library_scan_fanout_concurrency (Union[Unset, int]): Gets or sets the how the library scan fans out.
        library_metadata_refresh_concurrency (Union[Unset, int]): Gets or sets the how many metadata refreshes can run
            concurrently.
        remove_old_plugins (Union[Unset, bool]): Gets or sets a value indicating whether older plugins should
            automatically be deleted from the plugin folder.
        allow_client_log_upload (Union[Unset, bool]): Gets or sets a value indicating whether clients should be allowed
            to upload logs.
    """

    log_file_retention_days: Union[Unset, int] = UNSET
    is_startup_wizard_completed: Union[Unset, bool] = UNSET
    cache_path: Union[Unset, None, str] = UNSET
    previous_version: Union[Unset, None, str] = UNSET
    previous_version_str: Union[Unset, None, str] = UNSET
    enable_metrics: Union[Unset, bool] = UNSET
    enable_normalized_item_by_name_ids: Union[Unset, bool] = UNSET
    is_port_authorized: Union[Unset, bool] = UNSET
    quick_connect_available: Union[Unset, bool] = UNSET
    enable_case_sensitive_item_ids: Union[Unset, bool] = UNSET
    disable_live_tv_channel_user_data_name: Union[Unset, bool] = UNSET
    metadata_path: Union[Unset, str] = UNSET
    metadata_network_path: Union[Unset, str] = UNSET
    preferred_metadata_language: Union[Unset, str] = UNSET
    metadata_country_code: Union[Unset, str] = UNSET
    sort_replace_characters: Union[Unset, List[str]] = UNSET
    sort_remove_characters: Union[Unset, List[str]] = UNSET
    sort_remove_words: Union[Unset, List[str]] = UNSET
    min_resume_pct: Union[Unset, int] = UNSET
    max_resume_pct: Union[Unset, int] = UNSET
    min_resume_duration_seconds: Union[Unset, int] = UNSET
    min_audiobook_resume: Union[Unset, int] = UNSET
    max_audiobook_resume: Union[Unset, int] = UNSET
    library_monitor_delay: Union[Unset, int] = UNSET
    image_saving_convention: Union[Unset, ImageSavingConvention] = UNSET
    metadata_options: Union[Unset, List["MetadataOptions"]] = UNSET
    skip_deserialization_for_basic_types: Union[Unset, bool] = UNSET
    server_name: Union[Unset, str] = UNSET
    ui_culture: Union[Unset, str] = UNSET
    save_metadata_hidden: Union[Unset, bool] = UNSET
    content_types: Union[Unset, List["NameValuePair"]] = UNSET
    remote_client_bitrate_limit: Union[Unset, int] = UNSET
    enable_folder_view: Union[Unset, bool] = UNSET
    enable_grouping_into_collections: Union[Unset, bool] = UNSET
    display_specials_within_seasons: Union[Unset, bool] = UNSET
    codecs_used: Union[Unset, List[str]] = UNSET
    plugin_repositories: Union[Unset, List["RepositoryInfo"]] = UNSET
    enable_external_content_in_suggestions: Union[Unset, bool] = UNSET
    image_extraction_timeout_ms: Union[Unset, int] = UNSET
    path_substitutions: Union[Unset, List["PathSubstitution"]] = UNSET
    enable_slow_response_warning: Union[Unset, bool] = UNSET
    slow_response_threshold_ms: Union[Unset, int] = UNSET
    cors_hosts: Union[Unset, List[str]] = UNSET
    activity_log_retention_days: Union[Unset, None, int] = UNSET
    library_scan_fanout_concurrency: Union[Unset, int] = UNSET
    library_metadata_refresh_concurrency: Union[Unset, int] = UNSET
    remove_old_plugins: Union[Unset, bool] = UNSET
    allow_client_log_upload: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        log_file_retention_days = self.log_file_retention_days
        is_startup_wizard_completed = self.is_startup_wizard_completed
        cache_path = self.cache_path
        previous_version = self.previous_version
        previous_version_str = self.previous_version_str
        enable_metrics = self.enable_metrics
        enable_normalized_item_by_name_ids = self.enable_normalized_item_by_name_ids
        is_port_authorized = self.is_port_authorized
        quick_connect_available = self.quick_connect_available
        enable_case_sensitive_item_ids = self.enable_case_sensitive_item_ids
        disable_live_tv_channel_user_data_name = self.disable_live_tv_channel_user_data_name
        metadata_path = self.metadata_path
        metadata_network_path = self.metadata_network_path
        preferred_metadata_language = self.preferred_metadata_language
        metadata_country_code = self.metadata_country_code
        sort_replace_characters: Union[Unset, List[str]] = UNSET
        if not isinstance(self.sort_replace_characters, Unset):
            sort_replace_characters = self.sort_replace_characters

        sort_remove_characters: Union[Unset, List[str]] = UNSET
        if not isinstance(self.sort_remove_characters, Unset):
            sort_remove_characters = self.sort_remove_characters

        sort_remove_words: Union[Unset, List[str]] = UNSET
        if not isinstance(self.sort_remove_words, Unset):
            sort_remove_words = self.sort_remove_words

        min_resume_pct = self.min_resume_pct
        max_resume_pct = self.max_resume_pct
        min_resume_duration_seconds = self.min_resume_duration_seconds
        min_audiobook_resume = self.min_audiobook_resume
        max_audiobook_resume = self.max_audiobook_resume
        library_monitor_delay = self.library_monitor_delay
        image_saving_convention: Union[Unset, str] = UNSET
        if not isinstance(self.image_saving_convention, Unset):
            image_saving_convention = self.image_saving_convention.value

        metadata_options: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.metadata_options, Unset):
            metadata_options = []
            for metadata_options_item_data in self.metadata_options:
                metadata_options_item = metadata_options_item_data.to_dict()

                metadata_options.append(metadata_options_item)

        skip_deserialization_for_basic_types = self.skip_deserialization_for_basic_types
        server_name = self.server_name
        ui_culture = self.ui_culture
        save_metadata_hidden = self.save_metadata_hidden
        content_types: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.content_types, Unset):
            content_types = []
            for content_types_item_data in self.content_types:
                content_types_item = content_types_item_data.to_dict()

                content_types.append(content_types_item)

        remote_client_bitrate_limit = self.remote_client_bitrate_limit
        enable_folder_view = self.enable_folder_view
        enable_grouping_into_collections = self.enable_grouping_into_collections
        display_specials_within_seasons = self.display_specials_within_seasons
        codecs_used: Union[Unset, List[str]] = UNSET
        if not isinstance(self.codecs_used, Unset):
            codecs_used = self.codecs_used

        plugin_repositories: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.plugin_repositories, Unset):
            plugin_repositories = []
            for plugin_repositories_item_data in self.plugin_repositories:
                plugin_repositories_item = plugin_repositories_item_data.to_dict()

                plugin_repositories.append(plugin_repositories_item)

        enable_external_content_in_suggestions = self.enable_external_content_in_suggestions
        image_extraction_timeout_ms = self.image_extraction_timeout_ms
        path_substitutions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.path_substitutions, Unset):
            path_substitutions = []
            for path_substitutions_item_data in self.path_substitutions:
                path_substitutions_item = path_substitutions_item_data.to_dict()

                path_substitutions.append(path_substitutions_item)

        enable_slow_response_warning = self.enable_slow_response_warning
        slow_response_threshold_ms = self.slow_response_threshold_ms
        cors_hosts: Union[Unset, List[str]] = UNSET
        if not isinstance(self.cors_hosts, Unset):
            cors_hosts = self.cors_hosts

        activity_log_retention_days = self.activity_log_retention_days
        library_scan_fanout_concurrency = self.library_scan_fanout_concurrency
        library_metadata_refresh_concurrency = self.library_metadata_refresh_concurrency
        remove_old_plugins = self.remove_old_plugins
        allow_client_log_upload = self.allow_client_log_upload

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if log_file_retention_days is not UNSET:
            field_dict["LogFileRetentionDays"] = log_file_retention_days
        if is_startup_wizard_completed is not UNSET:
            field_dict["IsStartupWizardCompleted"] = is_startup_wizard_completed
        if cache_path is not UNSET:
            field_dict["CachePath"] = cache_path
        if previous_version is not UNSET:
            field_dict["PreviousVersion"] = previous_version
        if previous_version_str is not UNSET:
            field_dict["PreviousVersionStr"] = previous_version_str
        if enable_metrics is not UNSET:
            field_dict["EnableMetrics"] = enable_metrics
        if enable_normalized_item_by_name_ids is not UNSET:
            field_dict["EnableNormalizedItemByNameIds"] = enable_normalized_item_by_name_ids
        if is_port_authorized is not UNSET:
            field_dict["IsPortAuthorized"] = is_port_authorized
        if quick_connect_available is not UNSET:
            field_dict["QuickConnectAvailable"] = quick_connect_available
        if enable_case_sensitive_item_ids is not UNSET:
            field_dict["EnableCaseSensitiveItemIds"] = enable_case_sensitive_item_ids
        if disable_live_tv_channel_user_data_name is not UNSET:
            field_dict["DisableLiveTvChannelUserDataName"] = disable_live_tv_channel_user_data_name
        if metadata_path is not UNSET:
            field_dict["MetadataPath"] = metadata_path
        if metadata_network_path is not UNSET:
            field_dict["MetadataNetworkPath"] = metadata_network_path
        if preferred_metadata_language is not UNSET:
            field_dict["PreferredMetadataLanguage"] = preferred_metadata_language
        if metadata_country_code is not UNSET:
            field_dict["MetadataCountryCode"] = metadata_country_code
        if sort_replace_characters is not UNSET:
            field_dict["SortReplaceCharacters"] = sort_replace_characters
        if sort_remove_characters is not UNSET:
            field_dict["SortRemoveCharacters"] = sort_remove_characters
        if sort_remove_words is not UNSET:
            field_dict["SortRemoveWords"] = sort_remove_words
        if min_resume_pct is not UNSET:
            field_dict["MinResumePct"] = min_resume_pct
        if max_resume_pct is not UNSET:
            field_dict["MaxResumePct"] = max_resume_pct
        if min_resume_duration_seconds is not UNSET:
            field_dict["MinResumeDurationSeconds"] = min_resume_duration_seconds
        if min_audiobook_resume is not UNSET:
            field_dict["MinAudiobookResume"] = min_audiobook_resume
        if max_audiobook_resume is not UNSET:
            field_dict["MaxAudiobookResume"] = max_audiobook_resume
        if library_monitor_delay is not UNSET:
            field_dict["LibraryMonitorDelay"] = library_monitor_delay
        if image_saving_convention is not UNSET:
            field_dict["ImageSavingConvention"] = image_saving_convention
        if metadata_options is not UNSET:
            field_dict["MetadataOptions"] = metadata_options
        if skip_deserialization_for_basic_types is not UNSET:
            field_dict["SkipDeserializationForBasicTypes"] = skip_deserialization_for_basic_types
        if server_name is not UNSET:
            field_dict["ServerName"] = server_name
        if ui_culture is not UNSET:
            field_dict["UICulture"] = ui_culture
        if save_metadata_hidden is not UNSET:
            field_dict["SaveMetadataHidden"] = save_metadata_hidden
        if content_types is not UNSET:
            field_dict["ContentTypes"] = content_types
        if remote_client_bitrate_limit is not UNSET:
            field_dict["RemoteClientBitrateLimit"] = remote_client_bitrate_limit
        if enable_folder_view is not UNSET:
            field_dict["EnableFolderView"] = enable_folder_view
        if enable_grouping_into_collections is not UNSET:
            field_dict["EnableGroupingIntoCollections"] = enable_grouping_into_collections
        if display_specials_within_seasons is not UNSET:
            field_dict["DisplaySpecialsWithinSeasons"] = display_specials_within_seasons
        if codecs_used is not UNSET:
            field_dict["CodecsUsed"] = codecs_used
        if plugin_repositories is not UNSET:
            field_dict["PluginRepositories"] = plugin_repositories
        if enable_external_content_in_suggestions is not UNSET:
            field_dict["EnableExternalContentInSuggestions"] = enable_external_content_in_suggestions
        if image_extraction_timeout_ms is not UNSET:
            field_dict["ImageExtractionTimeoutMs"] = image_extraction_timeout_ms
        if path_substitutions is not UNSET:
            field_dict["PathSubstitutions"] = path_substitutions
        if enable_slow_response_warning is not UNSET:
            field_dict["EnableSlowResponseWarning"] = enable_slow_response_warning
        if slow_response_threshold_ms is not UNSET:
            field_dict["SlowResponseThresholdMs"] = slow_response_threshold_ms
        if cors_hosts is not UNSET:
            field_dict["CorsHosts"] = cors_hosts
        if activity_log_retention_days is not UNSET:
            field_dict["ActivityLogRetentionDays"] = activity_log_retention_days
        if library_scan_fanout_concurrency is not UNSET:
            field_dict["LibraryScanFanoutConcurrency"] = library_scan_fanout_concurrency
        if library_metadata_refresh_concurrency is not UNSET:
            field_dict["LibraryMetadataRefreshConcurrency"] = library_metadata_refresh_concurrency
        if remove_old_plugins is not UNSET:
            field_dict["RemoveOldPlugins"] = remove_old_plugins
        if allow_client_log_upload is not UNSET:
            field_dict["AllowClientLogUpload"] = allow_client_log_upload

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.metadata_options import MetadataOptions
        from ..models.name_value_pair import NameValuePair
        from ..models.path_substitution import PathSubstitution
        from ..models.repository_info import RepositoryInfo

        d = src_dict.copy()
        log_file_retention_days = d.pop("LogFileRetentionDays", UNSET)

        is_startup_wizard_completed = d.pop("IsStartupWizardCompleted", UNSET)

        cache_path = d.pop("CachePath", UNSET)

        previous_version = d.pop("PreviousVersion", UNSET)

        previous_version_str = d.pop("PreviousVersionStr", UNSET)

        enable_metrics = d.pop("EnableMetrics", UNSET)

        enable_normalized_item_by_name_ids = d.pop("EnableNormalizedItemByNameIds", UNSET)

        is_port_authorized = d.pop("IsPortAuthorized", UNSET)

        quick_connect_available = d.pop("QuickConnectAvailable", UNSET)

        enable_case_sensitive_item_ids = d.pop("EnableCaseSensitiveItemIds", UNSET)

        disable_live_tv_channel_user_data_name = d.pop("DisableLiveTvChannelUserDataName", UNSET)

        metadata_path = d.pop("MetadataPath", UNSET)

        metadata_network_path = d.pop("MetadataNetworkPath", UNSET)

        preferred_metadata_language = d.pop("PreferredMetadataLanguage", UNSET)

        metadata_country_code = d.pop("MetadataCountryCode", UNSET)

        sort_replace_characters = cast(List[str], d.pop("SortReplaceCharacters", UNSET))

        sort_remove_characters = cast(List[str], d.pop("SortRemoveCharacters", UNSET))

        sort_remove_words = cast(List[str], d.pop("SortRemoveWords", UNSET))

        min_resume_pct = d.pop("MinResumePct", UNSET)

        max_resume_pct = d.pop("MaxResumePct", UNSET)

        min_resume_duration_seconds = d.pop("MinResumeDurationSeconds", UNSET)

        min_audiobook_resume = d.pop("MinAudiobookResume", UNSET)

        max_audiobook_resume = d.pop("MaxAudiobookResume", UNSET)

        library_monitor_delay = d.pop("LibraryMonitorDelay", UNSET)

        _image_saving_convention = d.pop("ImageSavingConvention", UNSET)
        image_saving_convention: Union[Unset, ImageSavingConvention]
        if isinstance(_image_saving_convention, Unset):
            image_saving_convention = UNSET
        else:
            image_saving_convention = ImageSavingConvention(_image_saving_convention)

        metadata_options = []
        _metadata_options = d.pop("MetadataOptions", UNSET)
        for metadata_options_item_data in _metadata_options or []:
            metadata_options_item = MetadataOptions.from_dict(metadata_options_item_data)

            metadata_options.append(metadata_options_item)

        skip_deserialization_for_basic_types = d.pop("SkipDeserializationForBasicTypes", UNSET)

        server_name = d.pop("ServerName", UNSET)

        ui_culture = d.pop("UICulture", UNSET)

        save_metadata_hidden = d.pop("SaveMetadataHidden", UNSET)

        content_types = []
        _content_types = d.pop("ContentTypes", UNSET)
        for content_types_item_data in _content_types or []:
            content_types_item = NameValuePair.from_dict(content_types_item_data)

            content_types.append(content_types_item)

        remote_client_bitrate_limit = d.pop("RemoteClientBitrateLimit", UNSET)

        enable_folder_view = d.pop("EnableFolderView", UNSET)

        enable_grouping_into_collections = d.pop("EnableGroupingIntoCollections", UNSET)

        display_specials_within_seasons = d.pop("DisplaySpecialsWithinSeasons", UNSET)

        codecs_used = cast(List[str], d.pop("CodecsUsed", UNSET))

        plugin_repositories = []
        _plugin_repositories = d.pop("PluginRepositories", UNSET)
        for plugin_repositories_item_data in _plugin_repositories or []:
            plugin_repositories_item = RepositoryInfo.from_dict(plugin_repositories_item_data)

            plugin_repositories.append(plugin_repositories_item)

        enable_external_content_in_suggestions = d.pop("EnableExternalContentInSuggestions", UNSET)

        image_extraction_timeout_ms = d.pop("ImageExtractionTimeoutMs", UNSET)

        path_substitutions = []
        _path_substitutions = d.pop("PathSubstitutions", UNSET)
        for path_substitutions_item_data in _path_substitutions or []:
            path_substitutions_item = PathSubstitution.from_dict(path_substitutions_item_data)

            path_substitutions.append(path_substitutions_item)

        enable_slow_response_warning = d.pop("EnableSlowResponseWarning", UNSET)

        slow_response_threshold_ms = d.pop("SlowResponseThresholdMs", UNSET)

        cors_hosts = cast(List[str], d.pop("CorsHosts", UNSET))

        activity_log_retention_days = d.pop("ActivityLogRetentionDays", UNSET)

        library_scan_fanout_concurrency = d.pop("LibraryScanFanoutConcurrency", UNSET)

        library_metadata_refresh_concurrency = d.pop("LibraryMetadataRefreshConcurrency", UNSET)

        remove_old_plugins = d.pop("RemoveOldPlugins", UNSET)

        allow_client_log_upload = d.pop("AllowClientLogUpload", UNSET)

        server_configuration = cls(
            log_file_retention_days=log_file_retention_days,
            is_startup_wizard_completed=is_startup_wizard_completed,
            cache_path=cache_path,
            previous_version=previous_version,
            previous_version_str=previous_version_str,
            enable_metrics=enable_metrics,
            enable_normalized_item_by_name_ids=enable_normalized_item_by_name_ids,
            is_port_authorized=is_port_authorized,
            quick_connect_available=quick_connect_available,
            enable_case_sensitive_item_ids=enable_case_sensitive_item_ids,
            disable_live_tv_channel_user_data_name=disable_live_tv_channel_user_data_name,
            metadata_path=metadata_path,
            metadata_network_path=metadata_network_path,
            preferred_metadata_language=preferred_metadata_language,
            metadata_country_code=metadata_country_code,
            sort_replace_characters=sort_replace_characters,
            sort_remove_characters=sort_remove_characters,
            sort_remove_words=sort_remove_words,
            min_resume_pct=min_resume_pct,
            max_resume_pct=max_resume_pct,
            min_resume_duration_seconds=min_resume_duration_seconds,
            min_audiobook_resume=min_audiobook_resume,
            max_audiobook_resume=max_audiobook_resume,
            library_monitor_delay=library_monitor_delay,
            image_saving_convention=image_saving_convention,
            metadata_options=metadata_options,
            skip_deserialization_for_basic_types=skip_deserialization_for_basic_types,
            server_name=server_name,
            ui_culture=ui_culture,
            save_metadata_hidden=save_metadata_hidden,
            content_types=content_types,
            remote_client_bitrate_limit=remote_client_bitrate_limit,
            enable_folder_view=enable_folder_view,
            enable_grouping_into_collections=enable_grouping_into_collections,
            display_specials_within_seasons=display_specials_within_seasons,
            codecs_used=codecs_used,
            plugin_repositories=plugin_repositories,
            enable_external_content_in_suggestions=enable_external_content_in_suggestions,
            image_extraction_timeout_ms=image_extraction_timeout_ms,
            path_substitutions=path_substitutions,
            enable_slow_response_warning=enable_slow_response_warning,
            slow_response_threshold_ms=slow_response_threshold_ms,
            cors_hosts=cors_hosts,
            activity_log_retention_days=activity_log_retention_days,
            library_scan_fanout_concurrency=library_scan_fanout_concurrency,
            library_metadata_refresh_concurrency=library_metadata_refresh_concurrency,
            remove_old_plugins=remove_old_plugins,
            allow_client_log_upload=allow_client_log_upload,
        )

        return server_configuration



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.general_command_type import GeneralCommandType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.base_item import BaseItem
    from ..models.base_item_dto import BaseItemDto
    from ..models.client_capabilities import ClientCapabilities
    from ..models.player_state_info import PlayerStateInfo
    from ..models.queue_item import QueueItem
    from ..models.session_user_info import SessionUserInfo
    from ..models.transcoding_info import TranscodingInfo


T = TypeVar("T", bound="SessionInfo")


@_attrs_define
class SessionInfo:
    """Class SessionInfo.

    Attributes:
        play_state (Union[Unset, None, PlayerStateInfo]):
        additional_users (Union[Unset, None, List['SessionUserInfo']]):
        capabilities (Union[Unset, None, ClientCapabilities]):
        remote_end_point (Union[Unset, None, str]): Gets or sets the remote end point.
        playable_media_types (Union[Unset, None, List[str]]): Gets the playable media types.
        id (Union[Unset, None, str]): Gets or sets the id.
        user_id (Union[Unset, str]): Gets or sets the user id.
        user_name (Union[Unset, None, str]): Gets or sets the username.
        client (Union[Unset, None, str]): Gets or sets the type of the client.
        last_activity_date (Union[Unset, datetime.datetime]): Gets or sets the last activity date.
        last_playback_check_in (Union[Unset, datetime.datetime]): Gets or sets the last playback check in.
        device_name (Union[Unset, None, str]): Gets or sets the name of the device.
        device_type (Union[Unset, None, str]): Gets or sets the type of the device.
        now_playing_item (Union[Unset, None, BaseItemDto]): This is strictly used as a data transfer object from the api
            layer.
            This holds information about a BaseItem in a format that is convenient for the client.
        full_now_playing_item (Union[Unset, None, BaseItem]): Class BaseItem.
        now_viewing_item (Union[Unset, None, BaseItemDto]): This is strictly used as a data transfer object from the api
            layer.
            This holds information about a BaseItem in a format that is convenient for the client.
        device_id (Union[Unset, None, str]): Gets or sets the device id.
        application_version (Union[Unset, None, str]): Gets or sets the application version.
        transcoding_info (Union[Unset, None, TranscodingInfo]):
        is_active (Union[Unset, bool]): Gets a value indicating whether this instance is active.
        supports_media_control (Union[Unset, bool]):
        supports_remote_control (Union[Unset, bool]):
        now_playing_queue (Union[Unset, None, List['QueueItem']]):
        now_playing_queue_full_items (Union[Unset, None, List['BaseItemDto']]):
        has_custom_device_name (Union[Unset, bool]):
        playlist_item_id (Union[Unset, None, str]):
        server_id (Union[Unset, None, str]):
        user_primary_image_tag (Union[Unset, None, str]):
        supported_commands (Union[Unset, None, List[GeneralCommandType]]): Gets the supported commands.
    """

    play_state: Union[Unset, None, "PlayerStateInfo"] = UNSET
    additional_users: Union[Unset, None, List["SessionUserInfo"]] = UNSET
    capabilities: Union[Unset, None, "ClientCapabilities"] = UNSET
    remote_end_point: Union[Unset, None, str] = UNSET
    playable_media_types: Union[Unset, None, List[str]] = UNSET
    id: Union[Unset, None, str] = UNSET
    user_id: Union[Unset, str] = UNSET
    user_name: Union[Unset, None, str] = UNSET
    client: Union[Unset, None, str] = UNSET
    last_activity_date: Union[Unset, datetime.datetime] = UNSET
    last_playback_check_in: Union[Unset, datetime.datetime] = UNSET
    device_name: Union[Unset, None, str] = UNSET
    device_type: Union[Unset, None, str] = UNSET
    now_playing_item: Union[Unset, None, "BaseItemDto"] = UNSET
    full_now_playing_item: Union[Unset, None, "BaseItem"] = UNSET
    now_viewing_item: Union[Unset, None, "BaseItemDto"] = UNSET
    device_id: Union[Unset, None, str] = UNSET
    application_version: Union[Unset, None, str] = UNSET
    transcoding_info: Union[Unset, None, "TranscodingInfo"] = UNSET
    is_active: Union[Unset, bool] = UNSET
    supports_media_control: Union[Unset, bool] = UNSET
    supports_remote_control: Union[Unset, bool] = UNSET
    now_playing_queue: Union[Unset, None, List["QueueItem"]] = UNSET
    now_playing_queue_full_items: Union[Unset, None, List["BaseItemDto"]] = UNSET
    has_custom_device_name: Union[Unset, bool] = UNSET
    playlist_item_id: Union[Unset, None, str] = UNSET
    server_id: Union[Unset, None, str] = UNSET
    user_primary_image_tag: Union[Unset, None, str] = UNSET
    supported_commands: Union[Unset, None, List[GeneralCommandType]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        play_state: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.play_state, Unset):
            play_state = self.play_state.to_dict() if self.play_state else None

        additional_users: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.additional_users, Unset):
            if self.additional_users is None:
                additional_users = None
            else:
                additional_users = []
                for additional_users_item_data in self.additional_users:
                    additional_users_item = additional_users_item_data.to_dict()

                    additional_users.append(additional_users_item)

        capabilities: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.capabilities, Unset):
            capabilities = self.capabilities.to_dict() if self.capabilities else None

        remote_end_point = self.remote_end_point
        playable_media_types: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.playable_media_types, Unset):
            if self.playable_media_types is None:
                playable_media_types = None
            else:
                playable_media_types = self.playable_media_types

        id = self.id
        user_id = self.user_id
        user_name = self.user_name
        client = self.client
        last_activity_date: Union[Unset, str] = UNSET
        if not isinstance(self.last_activity_date, Unset):
            last_activity_date = self.last_activity_date.isoformat()

        last_playback_check_in: Union[Unset, str] = UNSET
        if not isinstance(self.last_playback_check_in, Unset):
            last_playback_check_in = self.last_playback_check_in.isoformat()

        device_name = self.device_name
        device_type = self.device_type
        now_playing_item: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.now_playing_item, Unset):
            now_playing_item = self.now_playing_item.to_dict() if self.now_playing_item else None

        full_now_playing_item: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.full_now_playing_item, Unset):
            full_now_playing_item = self.full_now_playing_item.to_dict() if self.full_now_playing_item else None

        now_viewing_item: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.now_viewing_item, Unset):
            now_viewing_item = self.now_viewing_item.to_dict() if self.now_viewing_item else None

        device_id = self.device_id
        application_version = self.application_version
        transcoding_info: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.transcoding_info, Unset):
            transcoding_info = self.transcoding_info.to_dict() if self.transcoding_info else None

        is_active = self.is_active
        supports_media_control = self.supports_media_control
        supports_remote_control = self.supports_remote_control
        now_playing_queue: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.now_playing_queue, Unset):
            if self.now_playing_queue is None:
                now_playing_queue = None
            else:
                now_playing_queue = []
                for now_playing_queue_item_data in self.now_playing_queue:
                    now_playing_queue_item = now_playing_queue_item_data.to_dict()

                    now_playing_queue.append(now_playing_queue_item)

        now_playing_queue_full_items: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.now_playing_queue_full_items, Unset):
            if self.now_playing_queue_full_items is None:
                now_playing_queue_full_items = None
            else:
                now_playing_queue_full_items = []
                for now_playing_queue_full_items_item_data in self.now_playing_queue_full_items:
                    now_playing_queue_full_items_item = now_playing_queue_full_items_item_data.to_dict()

                    now_playing_queue_full_items.append(now_playing_queue_full_items_item)

        has_custom_device_name = self.has_custom_device_name
        playlist_item_id = self.playlist_item_id
        server_id = self.server_id
        user_primary_image_tag = self.user_primary_image_tag
        supported_commands: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.supported_commands, Unset):
            if self.supported_commands is None:
                supported_commands = None
            else:
                supported_commands = []
                for supported_commands_item_data in self.supported_commands:
                    supported_commands_item = supported_commands_item_data.value

                    supported_commands.append(supported_commands_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if play_state is not UNSET:
            field_dict["PlayState"] = play_state
        if additional_users is not UNSET:
            field_dict["AdditionalUsers"] = additional_users
        if capabilities is not UNSET:
            field_dict["Capabilities"] = capabilities
        if remote_end_point is not UNSET:
            field_dict["RemoteEndPoint"] = remote_end_point
        if playable_media_types is not UNSET:
            field_dict["PlayableMediaTypes"] = playable_media_types
        if id is not UNSET:
            field_dict["Id"] = id
        if user_id is not UNSET:
            field_dict["UserId"] = user_id
        if user_name is not UNSET:
            field_dict["UserName"] = user_name
        if client is not UNSET:
            field_dict["Client"] = client
        if last_activity_date is not UNSET:
            field_dict["LastActivityDate"] = last_activity_date
        if last_playback_check_in is not UNSET:
            field_dict["LastPlaybackCheckIn"] = last_playback_check_in
        if device_name is not UNSET:
            field_dict["DeviceName"] = device_name
        if device_type is not UNSET:
            field_dict["DeviceType"] = device_type
        if now_playing_item is not UNSET:
            field_dict["NowPlayingItem"] = now_playing_item
        if full_now_playing_item is not UNSET:
            field_dict["FullNowPlayingItem"] = full_now_playing_item
        if now_viewing_item is not UNSET:
            field_dict["NowViewingItem"] = now_viewing_item
        if device_id is not UNSET:
            field_dict["DeviceId"] = device_id
        if application_version is not UNSET:
            field_dict["ApplicationVersion"] = application_version
        if transcoding_info is not UNSET:
            field_dict["TranscodingInfo"] = transcoding_info
        if is_active is not UNSET:
            field_dict["IsActive"] = is_active
        if supports_media_control is not UNSET:
            field_dict["SupportsMediaControl"] = supports_media_control
        if supports_remote_control is not UNSET:
            field_dict["SupportsRemoteControl"] = supports_remote_control
        if now_playing_queue is not UNSET:
            field_dict["NowPlayingQueue"] = now_playing_queue
        if now_playing_queue_full_items is not UNSET:
            field_dict["NowPlayingQueueFullItems"] = now_playing_queue_full_items
        if has_custom_device_name is not UNSET:
            field_dict["HasCustomDeviceName"] = has_custom_device_name
        if playlist_item_id is not UNSET:
            field_dict["PlaylistItemId"] = playlist_item_id
        if server_id is not UNSET:
            field_dict["ServerId"] = server_id
        if user_primary_image_tag is not UNSET:
            field_dict["UserPrimaryImageTag"] = user_primary_image_tag
        if supported_commands is not UNSET:
            field_dict["SupportedCommands"] = supported_commands

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.base_item import BaseItem
        from ..models.base_item_dto import BaseItemDto
        from ..models.client_capabilities import ClientCapabilities
        from ..models.player_state_info import PlayerStateInfo
        from ..models.queue_item import QueueItem
        from ..models.session_user_info import SessionUserInfo
        from ..models.transcoding_info import TranscodingInfo

        d = src_dict.copy()
        _play_state = d.pop("PlayState", UNSET)
        play_state: Union[Unset, None, PlayerStateInfo]
        if _play_state is None:
            play_state = None
        elif isinstance(_play_state, Unset):
            play_state = UNSET
        else:
            play_state = PlayerStateInfo.from_dict(_play_state)

        additional_users = []
        _additional_users = d.pop("AdditionalUsers", UNSET)
        for additional_users_item_data in _additional_users or []:
            additional_users_item = SessionUserInfo.from_dict(additional_users_item_data)

            additional_users.append(additional_users_item)

        _capabilities = d.pop("Capabilities", UNSET)
        capabilities: Union[Unset, None, ClientCapabilities]
        if _capabilities is None:
            capabilities = None
        elif isinstance(_capabilities, Unset):
            capabilities = UNSET
        else:
            capabilities = ClientCapabilities.from_dict(_capabilities)

        remote_end_point = d.pop("RemoteEndPoint", UNSET)

        playable_media_types = cast(List[str], d.pop("PlayableMediaTypes", UNSET))

        id = d.pop("Id", UNSET)

        user_id = d.pop("UserId", UNSET)

        user_name = d.pop("UserName", UNSET)

        client = d.pop("Client", UNSET)

        _last_activity_date = d.pop("LastActivityDate", UNSET)
        last_activity_date: Union[Unset, datetime.datetime]
        if isinstance(_last_activity_date, Unset):
            last_activity_date = UNSET
        else:
            last_activity_date = isoparse(_last_activity_date)

        _last_playback_check_in = d.pop("LastPlaybackCheckIn", UNSET)
        last_playback_check_in: Union[Unset, datetime.datetime]
        if isinstance(_last_playback_check_in, Unset):
            last_playback_check_in = UNSET
        else:
            last_playback_check_in = isoparse(_last_playback_check_in)

        device_name = d.pop("DeviceName", UNSET)

        device_type = d.pop("DeviceType", UNSET)

        _now_playing_item = d.pop("NowPlayingItem", UNSET)
        now_playing_item: Union[Unset, None, BaseItemDto]
        if _now_playing_item is None:
            now_playing_item = None
        elif isinstance(_now_playing_item, Unset):
            now_playing_item = UNSET
        else:
            now_playing_item = BaseItemDto.from_dict(_now_playing_item)

        _full_now_playing_item = d.pop("FullNowPlayingItem", UNSET)
        full_now_playing_item: Union[Unset, None, BaseItem]
        if _full_now_playing_item is None:
            full_now_playing_item = None
        elif isinstance(_full_now_playing_item, Unset):
            full_now_playing_item = UNSET
        else:
            full_now_playing_item = BaseItem.from_dict(_full_now_playing_item)

        _now_viewing_item = d.pop("NowViewingItem", UNSET)
        now_viewing_item: Union[Unset, None, BaseItemDto]
        if _now_viewing_item is None:
            now_viewing_item = None
        elif isinstance(_now_viewing_item, Unset):
            now_viewing_item = UNSET
        else:
            now_viewing_item = BaseItemDto.from_dict(_now_viewing_item)

        device_id = d.pop("DeviceId", UNSET)

        application_version = d.pop("ApplicationVersion", UNSET)

        _transcoding_info = d.pop("TranscodingInfo", UNSET)
        transcoding_info: Union[Unset, None, TranscodingInfo]
        if _transcoding_info is None:
            transcoding_info = None
        elif isinstance(_transcoding_info, Unset):
            transcoding_info = UNSET
        else:
            transcoding_info = TranscodingInfo.from_dict(_transcoding_info)

        is_active = d.pop("IsActive", UNSET)

        supports_media_control = d.pop("SupportsMediaControl", UNSET)

        supports_remote_control = d.pop("SupportsRemoteControl", UNSET)

        now_playing_queue = []
        _now_playing_queue = d.pop("NowPlayingQueue", UNSET)
        for now_playing_queue_item_data in _now_playing_queue or []:
            now_playing_queue_item = QueueItem.from_dict(now_playing_queue_item_data)

            now_playing_queue.append(now_playing_queue_item)

        now_playing_queue_full_items = []
        _now_playing_queue_full_items = d.pop("NowPlayingQueueFullItems", UNSET)
        for now_playing_queue_full_items_item_data in _now_playing_queue_full_items or []:
            now_playing_queue_full_items_item = BaseItemDto.from_dict(now_playing_queue_full_items_item_data)

            now_playing_queue_full_items.append(now_playing_queue_full_items_item)

        has_custom_device_name = d.pop("HasCustomDeviceName", UNSET)

        playlist_item_id = d.pop("PlaylistItemId", UNSET)

        server_id = d.pop("ServerId", UNSET)

        user_primary_image_tag = d.pop("UserPrimaryImageTag", UNSET)

        supported_commands = []
        _supported_commands = d.pop("SupportedCommands", UNSET)
        for supported_commands_item_data in _supported_commands or []:
            supported_commands_item = GeneralCommandType(supported_commands_item_data)

            supported_commands.append(supported_commands_item)

        session_info = cls(
            play_state=play_state,
            additional_users=additional_users,
            capabilities=capabilities,
            remote_end_point=remote_end_point,
            playable_media_types=playable_media_types,
            id=id,
            user_id=user_id,
            user_name=user_name,
            client=client,
            last_activity_date=last_activity_date,
            last_playback_check_in=last_playback_check_in,
            device_name=device_name,
            device_type=device_type,
            now_playing_item=now_playing_item,
            full_now_playing_item=full_now_playing_item,
            now_viewing_item=now_viewing_item,
            device_id=device_id,
            application_version=application_version,
            transcoding_info=transcoding_info,
            is_active=is_active,
            supports_media_control=supports_media_control,
            supports_remote_control=supports_remote_control,
            now_playing_queue=now_playing_queue,
            now_playing_queue_full_items=now_playing_queue_full_items,
            has_custom_device_name=has_custom_device_name,
            playlist_item_id=playlist_item_id,
            server_id=server_id,
            user_primary_image_tag=user_primary_image_tag,
            supported_commands=supported_commands,
        )

        return session_info



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="MediaUpdateInfoPathDto")


@_attrs_define
class MediaUpdateInfoPathDto:
    """The media update info path.

    Attributes:
        path (Union[Unset, None, str]): Gets or sets media path.
        update_type (Union[Unset, None, str]): Gets or sets media update type.
            Created, Modified, Deleted.
    """

    path: Union[Unset, None, str] = UNSET
    update_type: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        path = self.path
        update_type = self.update_type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if path is not UNSET:
            field_dict["Path"] = path
        if update_type is not UNSET:
            field_dict["UpdateType"] = update_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        path = d.pop("Path", UNSET)

        update_type = d.pop("UpdateType", UNSET)

        media_update_info_path_dto = cls(
            path=path,
            update_type=update_type,
        )

        return media_update_info_path_dto



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..models.hardware_encoding_type import HardwareEncodingType
from ..models.transcode_reason import TranscodeReason
from ..types import UNSET, Unset

T = TypeVar("T", bound="TranscodingInfo")


@_attrs_define
class TranscodingInfo:
    """
    Attributes:
        audio_codec (Union[Unset, None, str]):
        video_codec (Union[Unset, None, str]):
        container (Union[Unset, None, str]):
        is_video_direct (Union[Unset, bool]):
        is_audio_direct (Union[Unset, bool]):
        bitrate (Union[Unset, None, int]):
        framerate (Union[Unset, None, float]):
        completion_percentage (Union[Unset, None, float]):
        width (Union[Unset, None, int]):
        height (Union[Unset, None, int]):
        audio_channels (Union[Unset, None, int]):
        hardware_acceleration_type (Union[Unset, None, HardwareEncodingType]): Enum HardwareEncodingType.
        transcode_reasons (Union[Unset, List[TranscodeReason]]):
    """

    audio_codec: Union[Unset, None, str] = UNSET
    video_codec: Union[Unset, None, str] = UNSET
    container: Union[Unset, None, str] = UNSET
    is_video_direct: Union[Unset, bool] = UNSET
    is_audio_direct: Union[Unset, bool] = UNSET
    bitrate: Union[Unset, None, int] = UNSET
    framerate: Union[Unset, None, float] = UNSET
    completion_percentage: Union[Unset, None, float] = UNSET
    width: Union[Unset, None, int] = UNSET
    height: Union[Unset, None, int] = UNSET
    audio_channels: Union[Unset, None, int] = UNSET
    hardware_acceleration_type: Union[Unset, None, HardwareEncodingType] = UNSET
    transcode_reasons: Union[Unset, List[TranscodeReason]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        audio_codec = self.audio_codec
        video_codec = self.video_codec
        container = self.container
        is_video_direct = self.is_video_direct
        is_audio_direct = self.is_audio_direct
        bitrate = self.bitrate
        framerate = self.framerate
        completion_percentage = self.completion_percentage
        width = self.width
        height = self.height
        audio_channels = self.audio_channels
        hardware_acceleration_type: Union[Unset, None, str] = UNSET
        if not isinstance(self.hardware_acceleration_type, Unset):
            hardware_acceleration_type = (
                self.hardware_acceleration_type.value if self.hardware_acceleration_type else None
            )

        transcode_reasons: Union[Unset, List[str]] = UNSET
        if not isinstance(self.transcode_reasons, Unset):
            transcode_reasons = []
            for transcode_reasons_item_data in self.transcode_reasons:
                transcode_reasons_item = transcode_reasons_item_data.value

                transcode_reasons.append(transcode_reasons_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if audio_codec is not UNSET:
            field_dict["AudioCodec"] = audio_codec
        if video_codec is not UNSET:
            field_dict["VideoCodec"] = video_codec
        if container is not UNSET:
            field_dict["Container"] = container
        if is_video_direct is not UNSET:
            field_dict["IsVideoDirect"] = is_video_direct
        if is_audio_direct is not UNSET:
            field_dict["IsAudioDirect"] = is_audio_direct
        if bitrate is not UNSET:
            field_dict["Bitrate"] = bitrate
        if framerate is not UNSET:
            field_dict["Framerate"] = framerate
        if completion_percentage is not UNSET:
            field_dict["CompletionPercentage"] = completion_percentage
        if width is not UNSET:
            field_dict["Width"] = width
        if height is not UNSET:
            field_dict["Height"] = height
        if audio_channels is not UNSET:
            field_dict["AudioChannels"] = audio_channels
        if hardware_acceleration_type is not UNSET:
            field_dict["HardwareAccelerationType"] = hardware_acceleration_type
        if transcode_reasons is not UNSET:
            field_dict["TranscodeReasons"] = transcode_reasons

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        audio_codec = d.pop("AudioCodec", UNSET)

        video_codec = d.pop("VideoCodec", UNSET)

        container = d.pop("Container", UNSET)

        is_video_direct = d.pop("IsVideoDirect", UNSET)

        is_audio_direct = d.pop("IsAudioDirect", UNSET)

        bitrate = d.pop("Bitrate", UNSET)

        framerate = d.pop("Framerate", UNSET)

        completion_percentage = d.pop("CompletionPercentage", UNSET)

        width = d.pop("Width", UNSET)

        height = d.pop("Height", UNSET)

        audio_channels = d.pop("AudioChannels", UNSET)

        _hardware_acceleration_type = d.pop("HardwareAccelerationType", UNSET)
        hardware_acceleration_type: Union[Unset, None, HardwareEncodingType]
        if _hardware_acceleration_type is None:
            hardware_acceleration_type = None
        elif isinstance(_hardware_acceleration_type, Unset):
            hardware_acceleration_type = UNSET
        else:
            hardware_acceleration_type = HardwareEncodingType(_hardware_acceleration_type)

        transcode_reasons = []
        _transcode_reasons = d.pop("TranscodeReasons", UNSET)
        for transcode_reasons_item_data in _transcode_reasons or []:
            transcode_reasons_item = TranscodeReason(transcode_reasons_item_data)

            transcode_reasons.append(transcode_reasons_item)

        transcoding_info = cls(
            audio_codec=audio_codec,
            video_codec=video_codec,
            container=container,
            is_video_direct=is_video_direct,
            is_audio_direct=is_audio_direct,
            bitrate=bitrate,
            framerate=framerate,
            completion_percentage=completion_percentage,
            width=width,
            height=height,
            audio_channels=audio_channels,
            hardware_acceleration_type=hardware_acceleration_type,
            transcode_reasons=transcode_reasons,
        )

        return transcoding_info



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="MediaEncoderPathDto")


@_attrs_define
class MediaEncoderPathDto:
    """Media Encoder Path Dto.

    Attributes:
        path (Union[Unset, str]): Gets or sets media encoder path.
        path_type (Union[Unset, str]): Gets or sets media encoder path type.
    """

    path: Union[Unset, str] = UNSET
    path_type: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        path = self.path
        path_type = self.path_type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if path is not UNSET:
            field_dict["Path"] = path
        if path_type is not UNSET:
            field_dict["PathType"] = path_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        path = d.pop("Path", UNSET)

        path_type = d.pop("PathType", UNSET)

        media_encoder_path_dto = cls(
            path=path,
            path_type=path_type,
        )

        return media_encoder_path_dto



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="PingRequestDto")


@_attrs_define
class PingRequestDto:
    """Class PingRequestDto.

    Attributes:
        ping (Union[Unset, int]): Gets or sets the ping time.
    """

    ping: Union[Unset, int] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        ping = self.ping

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if ping is not UNSET:
            field_dict["Ping"] = ping

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        ping = d.pop("Ping", UNSET)

        ping_request_dto = cls(
            ping=ping,
        )

        return ping_request_dto



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.base_item_dto_query_result import BaseItemDtoQueryResult
from ...models.base_item_kind import BaseItemKind
from ...types import UNSET, Response, Unset


def _get_kwargs(
    user_id: str,
    *,
    media_type: Union[Unset, None, List[str]] = UNSET,
    type: Union[Unset, None, List[BaseItemKind]] = UNSET,
    start_index: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    enable_total_record_count: Union[Unset, None, bool] = False,
) -> Dict[str, Any]:
    pass

    params: Dict[str, Any] = {}
    json_media_type: Union[Unset, None, List[str]] = UNSET
    if not isinstance(media_type, Unset):
        if media_type is None:
            json_media_type = None
        else:
            json_media_type = media_type

    params["mediaType"] = json_media_type

    json_type: Union[Unset, None, List[str]] = UNSET
    if not isinstance(type, Unset):
        if type is None:
            json_type = None
        else:
            json_type = []
            for type_item_data in type:
                type_item = type_item_data.value

                json_type.append(type_item)

    params["type"] = json_type

    params["startIndex"] = start_index

    params["limit"] = limit

    params["enableTotalRecordCount"] = enable_total_record_count

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/Users/{userId}/Suggestions".format(
            userId=user_id,
        ),
        "params": params,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, BaseItemDtoQueryResult]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = BaseItemDtoQueryResult.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = cast(Any, None)
        return response_403
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, BaseItemDtoQueryResult]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    user_id: str,
    *,
    client: AuthenticatedClient,
    media_type: Union[Unset, None, List[str]] = UNSET,
    type: Union[Unset, None, List[BaseItemKind]] = UNSET,
    start_index: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    enable_total_record_count: Union[Unset, None, bool] = False,
) -> Response[Union[Any, BaseItemDtoQueryResult]]:
    """Gets suggestions.

    Args:
        user_id (str):
        media_type (Union[Unset, None, List[str]]):
        type (Union[Unset, None, List[BaseItemKind]]):
        start_index (Union[Unset, None, int]):
        limit (Union[Unset, None, int]):
        enable_total_record_count (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, BaseItemDtoQueryResult]]
    """

    kwargs = _get_kwargs(
        user_id=user_id,
        media_type=media_type,
        type=type,
        start_index=start_index,
        limit=limit,
        enable_total_record_count=enable_total_record_count,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    user_id: str,
    *,
    client: AuthenticatedClient,
    media_type: Union[Unset, None, List[str]] = UNSET,
    type: Union[Unset, None, List[BaseItemKind]] = UNSET,
    start_index: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    enable_total_record_count: Union[Unset, None, bool] = False,
) -> Optional[Union[Any, BaseItemDtoQueryResult]]:
    """Gets suggestions.

    Args:
        user_id (str):
        media_type (Union[Unset, None, List[str]]):
        type (Union[Unset, None, List[BaseItemKind]]):
        start_index (Union[Unset, None, int]):
        limit (Union[Unset, None, int]):
        enable_total_record_count (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, BaseItemDtoQueryResult]
    """

    return sync_detailed(
        user_id=user_id,
        client=client,
        media_type=media_type,
        type=type,
        start_index=start_index,
        limit=limit,
        enable_total_record_count=enable_total_record_count,
    ).parsed


async def asyncio_detailed(
    user_id: str,
    *,
    client: AuthenticatedClient,
    media_type: Union[Unset, None, List[str]] = UNSET,
    type: Union[Unset, None, List[BaseItemKind]] = UNSET,
    start_index: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    enable_total_record_count: Union[Unset, None, bool] = False,
) -> Response[Union[Any, BaseItemDtoQueryResult]]:
    """Gets suggestions.

    Args:
        user_id (str):
        media_type (Union[Unset, None, List[str]]):
        type (Union[Unset, None, List[BaseItemKind]]):
        start_index (Union[Unset, None, int]):
        limit (Union[Unset, None, int]):
        enable_total_record_count (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, BaseItemDtoQueryResult]]
    """

    kwargs = _get_kwargs(
        user_id=user_id,
        media_type=media_type,
        type=type,
        start_index=start_index,
        limit=limit,
        enable_total_record_count=enable_total_record_count,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    user_id: str,
    *,
    client: AuthenticatedClient,
    media_type: Union[Unset, None, List[str]] = UNSET,
    type: Union[Unset, None, List[BaseItemKind]] = UNSET,
    start_index: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    enable_total_record_count: Union[Unset, None, bool] = False,
) -> Optional[Union[Any, BaseItemDtoQueryResult]]:
    """Gets suggestions.

    Args:
        user_id (str):
        media_type (Union[Unset, None, List[str]]):
        type (Union[Unset, None, List[BaseItemKind]]):
        start_index (Union[Unset, None, int]):
        limit (Union[Unset, None, int]):
        enable_total_record_count (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, BaseItemDtoQueryResult]
    """

    return (
        await asyncio_detailed(
            user_id=user_id,
            client=client,
            media_type=media_type,
            type=type,
            start_index=start_index,
            limit=limit,
            enable_total_record_count=enable_total_record_count,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..models.dynamic_day_of_week import DynamicDayOfWeek
from ..types import UNSET, Unset

T = TypeVar("T", bound="AccessSchedule")


@_attrs_define
class AccessSchedule:
    """An entity representing a user's access schedule.

    Attributes:
        id (Union[Unset, int]): Gets the id of this instance.
        user_id (Union[Unset, str]): Gets the id of the associated user.
        day_of_week (Union[Unset, DynamicDayOfWeek]): An enum that represents a day of the week, weekdays, weekends, or
            all days.
        start_hour (Union[Unset, float]): Gets or sets the start hour.
        end_hour (Union[Unset, float]): Gets or sets the end hour.
    """

    id: Union[Unset, int] = UNSET
    user_id: Union[Unset, str] = UNSET
    day_of_week: Union[Unset, DynamicDayOfWeek] = UNSET
    start_hour: Union[Unset, float] = UNSET
    end_hour: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        user_id = self.user_id
        day_of_week: Union[Unset, str] = UNSET
        if not isinstance(self.day_of_week, Unset):
            day_of_week = self.day_of_week.value

        start_hour = self.start_hour
        end_hour = self.end_hour

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if id is not UNSET:
            field_dict["Id"] = id
        if user_id is not UNSET:
            field_dict["UserId"] = user_id
        if day_of_week is not UNSET:
            field_dict["DayOfWeek"] = day_of_week
        if start_hour is not UNSET:
            field_dict["StartHour"] = start_hour
        if end_hour is not UNSET:
            field_dict["EndHour"] = end_hour

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("Id", UNSET)

        user_id = d.pop("UserId", UNSET)

        _day_of_week = d.pop("DayOfWeek", UNSET)
        day_of_week: Union[Unset, DynamicDayOfWeek]
        if isinstance(_day_of_week, Unset):
            day_of_week = UNSET
        else:
            day_of_week = DynamicDayOfWeek(_day_of_week)

        start_hour = d.pop("StartHour", UNSET)

        end_hour = d.pop("EndHour", UNSET)

        access_schedule = cls(
            id=id,
            user_id=user_id,
            day_of_week=day_of_week,
            start_hour=start_hour,
            end_hour=end_hour,
        )

        return access_schedule



import datetime
from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.base_item_dto_query_result import BaseItemDtoQueryResult
from ...models.image_type import ImageType
from ...models.item_fields import ItemFields
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    user_id: Union[Unset, None, str] = UNSET,
    start_index: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    fields: Union[Unset, None, List[ItemFields]] = UNSET,
    series_id: Union[Unset, None, str] = UNSET,
    parent_id: Union[Unset, None, str] = UNSET,
    enable_images: Union[Unset, None, bool] = UNSET,
    image_type_limit: Union[Unset, None, int] = UNSET,
    enable_image_types: Union[Unset, None, List[ImageType]] = UNSET,
    enable_user_data: Union[Unset, None, bool] = UNSET,
    next_up_date_cutoff: Union[Unset, None, datetime.datetime] = UNSET,
    enable_total_record_count: Union[Unset, None, bool] = True,
    disable_first_episode: Union[Unset, None, bool] = False,
    enable_rewatching: Union[Unset, None, bool] = False,
) -> Dict[str, Any]:
    pass

    params: Dict[str, Any] = {}
    params["userId"] = user_id

    params["startIndex"] = start_index

    params["limit"] = limit

    json_fields: Union[Unset, None, List[str]] = UNSET
    if not isinstance(fields, Unset):
        if fields is None:
            json_fields = None
        else:
            json_fields = []
            for fields_item_data in fields:
                fields_item = fields_item_data.value

                json_fields.append(fields_item)

    params["fields"] = json_fields

    params["seriesId"] = series_id

    params["parentId"] = parent_id

    params["enableImages"] = enable_images

    params["imageTypeLimit"] = image_type_limit

    json_enable_image_types: Union[Unset, None, List[str]] = UNSET
    if not isinstance(enable_image_types, Unset):
        if enable_image_types is None:
            json_enable_image_types = None
        else:
            json_enable_image_types = []
            for enable_image_types_item_data in enable_image_types:
                enable_image_types_item = enable_image_types_item_data.value

                json_enable_image_types.append(enable_image_types_item)

    params["enableImageTypes"] = json_enable_image_types

    params["enableUserData"] = enable_user_data

    json_next_up_date_cutoff: Union[Unset, None, str] = UNSET
    if not isinstance(next_up_date_cutoff, Unset):
        json_next_up_date_cutoff = next_up_date_cutoff.isoformat() if next_up_date_cutoff else None

    params["nextUpDateCutoff"] = json_next_up_date_cutoff

    params["enableTotalRecordCount"] = enable_total_record_count

    params["disableFirstEpisode"] = disable_first_episode

    params["enableRewatching"] = enable_rewatching

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/Shows/NextUp",
        "params": params,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, BaseItemDtoQueryResult]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = BaseItemDtoQueryResult.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = cast(Any, None)
        return response_403
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, BaseItemDtoQueryResult]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    user_id: Union[Unset, None, str] = UNSET,
    start_index: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    fields: Union[Unset, None, List[ItemFields]] = UNSET,
    series_id: Union[Unset, None, str] = UNSET,
    parent_id: Union[Unset, None, str] = UNSET,
    enable_images: Union[Unset, None, bool] = UNSET,
    image_type_limit: Union[Unset, None, int] = UNSET,
    enable_image_types: Union[Unset, None, List[ImageType]] = UNSET,
    enable_user_data: Union[Unset, None, bool] = UNSET,
    next_up_date_cutoff: Union[Unset, None, datetime.datetime] = UNSET,
    enable_total_record_count: Union[Unset, None, bool] = True,
    disable_first_episode: Union[Unset, None, bool] = False,
    enable_rewatching: Union[Unset, None, bool] = False,
) -> Response[Union[Any, BaseItemDtoQueryResult]]:
    """Gets a list of next up episodes.

    Args:
        user_id (Union[Unset, None, str]):
        start_index (Union[Unset, None, int]):
        limit (Union[Unset, None, int]):
        fields (Union[Unset, None, List[ItemFields]]):
        series_id (Union[Unset, None, str]):
        parent_id (Union[Unset, None, str]):
        enable_images (Union[Unset, None, bool]):
        image_type_limit (Union[Unset, None, int]):
        enable_image_types (Union[Unset, None, List[ImageType]]):
        enable_user_data (Union[Unset, None, bool]):
        next_up_date_cutoff (Union[Unset, None, datetime.datetime]):
        enable_total_record_count (Union[Unset, None, bool]):  Default: True.
        disable_first_episode (Union[Unset, None, bool]):
        enable_rewatching (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, BaseItemDtoQueryResult]]
    """

    kwargs = _get_kwargs(
        user_id=user_id,
        start_index=start_index,
        limit=limit,
        fields=fields,
        series_id=series_id,
        parent_id=parent_id,
        enable_images=enable_images,
        image_type_limit=image_type_limit,
        enable_image_types=enable_image_types,
        enable_user_data=enable_user_data,
        next_up_date_cutoff=next_up_date_cutoff,
        enable_total_record_count=enable_total_record_count,
        disable_first_episode=disable_first_episode,
        enable_rewatching=enable_rewatching,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    user_id: Union[Unset, None, str] = UNSET,
    start_index: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    fields: Union[Unset, None, List[ItemFields]] = UNSET,
    series_id: Union[Unset, None, str] = UNSET,
    parent_id: Union[Unset, None, str] = UNSET,
    enable_images: Union[Unset, None, bool] = UNSET,
    image_type_limit: Union[Unset, None, int] = UNSET,
    enable_image_types: Union[Unset, None, List[ImageType]] = UNSET,
    enable_user_data: Union[Unset, None, bool] = UNSET,
    next_up_date_cutoff: Union[Unset, None, datetime.datetime] = UNSET,
    enable_total_record_count: Union[Unset, None, bool] = True,
    disable_first_episode: Union[Unset, None, bool] = False,
    enable_rewatching: Union[Unset, None, bool] = False,
) -> Optional[Union[Any, BaseItemDtoQueryResult]]:
    """Gets a list of next up episodes.

    Args:
        user_id (Union[Unset, None, str]):
        start_index (Union[Unset, None, int]):
        limit (Union[Unset, None, int]):
        fields (Union[Unset, None, List[ItemFields]]):
        series_id (Union[Unset, None, str]):
        parent_id (Union[Unset, None, str]):
        enable_images (Union[Unset, None, bool]):
        image_type_limit (Union[Unset, None, int]):
        enable_image_types (Union[Unset, None, List[ImageType]]):
        enable_user_data (Union[Unset, None, bool]):
        next_up_date_cutoff (Union[Unset, None, datetime.datetime]):
        enable_total_record_count (Union[Unset, None, bool]):  Default: True.
        disable_first_episode (Union[Unset, None, bool]):
        enable_rewatching (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, BaseItemDtoQueryResult]
    """

    return sync_detailed(
        client=client,
        user_id=user_id,
        start_index=start_index,
        limit=limit,
        fields=fields,
        series_id=series_id,
        parent_id=parent_id,
        enable_images=enable_images,
        image_type_limit=image_type_limit,
        enable_image_types=enable_image_types,
        enable_user_data=enable_user_data,
        next_up_date_cutoff=next_up_date_cutoff,
        enable_total_record_count=enable_total_record_count,
        disable_first_episode=disable_first_episode,
        enable_rewatching=enable_rewatching,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    user_id: Union[Unset, None, str] = UNSET,
    start_index: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    fields: Union[Unset, None, List[ItemFields]] = UNSET,
    series_id: Union[Unset, None, str] = UNSET,
    parent_id: Union[Unset, None, str] = UNSET,
    enable_images: Union[Unset, None, bool] = UNSET,
    image_type_limit: Union[Unset, None, int] = UNSET,
    enable_image_types: Union[Unset, None, List[ImageType]] = UNSET,
    enable_user_data: Union[Unset, None, bool] = UNSET,
    next_up_date_cutoff: Union[Unset, None, datetime.datetime] = UNSET,
    enable_total_record_count: Union[Unset, None, bool] = True,
    disable_first_episode: Union[Unset, None, bool] = False,
    enable_rewatching: Union[Unset, None, bool] = False,
) -> Response[Union[Any, BaseItemDtoQueryResult]]:
    """Gets a list of next up episodes.

    Args:
        user_id (Union[Unset, None, str]):
        start_index (Union[Unset, None, int]):
        limit (Union[Unset, None, int]):
        fields (Union[Unset, None, List[ItemFields]]):
        series_id (Union[Unset, None, str]):
        parent_id (Union[Unset, None, str]):
        enable_images (Union[Unset, None, bool]):
        image_type_limit (Union[Unset, None, int]):
        enable_image_types (Union[Unset, None, List[ImageType]]):
        enable_user_data (Union[Unset, None, bool]):
        next_up_date_cutoff (Union[Unset, None, datetime.datetime]):
        enable_total_record_count (Union[Unset, None, bool]):  Default: True.
        disable_first_episode (Union[Unset, None, bool]):
        enable_rewatching (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, BaseItemDtoQueryResult]]
    """

    kwargs = _get_kwargs(
        user_id=user_id,
        start_index=start_index,
        limit=limit,
        fields=fields,
        series_id=series_id,
        parent_id=parent_id,
        enable_images=enable_images,
        image_type_limit=image_type_limit,
        enable_image_types=enable_image_types,
        enable_user_data=enable_user_data,
        next_up_date_cutoff=next_up_date_cutoff,
        enable_total_record_count=enable_total_record_count,
        disable_first_episode=disable_first_episode,
        enable_rewatching=enable_rewatching,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    user_id: Union[Unset, None, str] = UNSET,
    start_index: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    fields: Union[Unset, None, List[ItemFields]] = UNSET,
    series_id: Union[Unset, None, str] = UNSET,
    parent_id: Union[Unset, None, str] = UNSET,
    enable_images: Union[Unset, None, bool] = UNSET,
    image_type_limit: Union[Unset, None, int] = UNSET,
    enable_image_types: Union[Unset, None, List[ImageType]] = UNSET,
    enable_user_data: Union[Unset, None, bool] = UNSET,
    next_up_date_cutoff: Union[Unset, None, datetime.datetime] = UNSET,
    enable_total_record_count: Union[Unset, None, bool] = True,
    disable_first_episode: Union[Unset, None, bool] = False,
    enable_rewatching: Union[Unset, None, bool] = False,
) -> Optional[Union[Any, BaseItemDtoQueryResult]]:
    """Gets a list of next up episodes.

    Args:
        user_id (Union[Unset, None, str]):
        start_index (Union[Unset, None, int]):
        limit (Union[Unset, None, int]):
        fields (Union[Unset, None, List[ItemFields]]):
        series_id (Union[Unset, None, str]):
        parent_id (Union[Unset, None, str]):
        enable_images (Union[Unset, None, bool]):
        image_type_limit (Union[Unset, None, int]):
        enable_image_types (Union[Unset, None, List[ImageType]]):
        enable_user_data (Union[Unset, None, bool]):
        next_up_date_cutoff (Union[Unset, None, datetime.datetime]):
        enable_total_record_count (Union[Unset, None, bool]):  Default: True.
        disable_first_episode (Union[Unset, None, bool]):
        enable_rewatching (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, BaseItemDtoQueryResult]
    """

    return (
        await asyncio_detailed(
            client=client,
            user_id=user_id,
            start_index=start_index,
            limit=limit,
            fields=fields,
            series_id=series_id,
            parent_id=parent_id,
            enable_images=enable_images,
            image_type_limit=image_type_limit,
            enable_image_types=enable_image_types,
            enable_user_data=enable_user_data,
            next_up_date_cutoff=next_up_date_cutoff,
            enable_total_record_count=enable_total_record_count,
            disable_first_episode=disable_first_episode,
            enable_rewatching=enable_rewatching,
        )
    ).parsed



from enum import Enum


class ProgramAudio(str, Enum):
    ATMOS = "Atmos"
    DOLBY = "Dolby"
    DOLBYDIGITAL = "DolbyDigital"
    MONO = "Mono"
    STEREO = "Stereo"
    THX = "Thx"

    def __str__(self) -> str:
        return str(self.value)
